/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum;

import java.util.*;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.lang.reflect.Method;
import com.eknowlogie.util.Strings;

/** <p>This class is used to store reflective information about a class 
    as needed by Gaviidae to manage its presentation.  This information 
    includes the potentially presentable elementary and composite fields 
    of the class.  A <b>presentable</b> field is one that can potentially 
    be presented to the user by Gaviidae during the presentation of its 
    enclosing object.  Currently, a presentable field is a public field or 
    a non-public field with a public getter and setter in its declaring class. 
    A presentable field may not always be presented to the user depending on 
    preference settings affecting it.  This class is not static and instantiated 
    by the session factory, because the determination of elementary and composite 
    fields is based on whether the field's type is being managed with an 
    elementary UI or not, and such preferences still depends on the factory 
    instantiation.</p> 

    @author Romel Rivera, eKnowlogie. Inc.
*/

public class Classes {
	
   /*-------------------------- Begin Fields --------------------------*/
	
   /** <p>A map from a class to its presentable fields.  This table's entries are 
       defined as follows: (Class anyClass, Vector of Field fields).</p>
   */
	
   protected Hashtable fields = new Hashtable ();

   /** <p>A map from a class to its elementary fields.  A field's type is 
       elementary if if its presentation is being managed with an ElementaryUI 
       class.  It is required that all primitive types be managed with ElementaryUI 
       classes. This table's entries are defined as follows: (Class anyClass, 
       Vector of Field elementaryFields).</p>
   */
   
   protected Hashtable elementaryFields = new Hashtable ();
   
   /** <p>A map from a class to its composite fields.  A field's type is composite 
       if its presentation is not being managed with an ElementaryUI class.  This 
       table's entries are defined as follows: (Class anyClass, Vector of Field 
       compositeFields).</p>
   */
   
   protected Hashtable compositeFields = new Hashtable ();
   
   /** <p>This is the UI factory associated with this instance.  It is 
       used to consult its preferences to determine if fields are elementary 
       or composite.</p> */
   
   protected UIFactory uif;
   
   /** <p>Convenience constant to use during reflection operations.</p> */
   
   private static Class [] parameterless = new Class [] {};
   
   /*--------------------------- End Fields ---------------------------*/

   /*----------------------- Begin Constructors -----------------------*/
   
   /** <p>This constructor associates this new instance with a given 
       UIFactory "uif", which will be used to select preferences when 
       determining if a field is elementary or not.</p>
   */
   
   public Classes (UIFactory uif) {
      this. uif = uif;
   }
   
   /*------------------------ End Constructors ------------------------*/

   /*---------------------- Begin Static Methods ----------------------*/
   
   /** <p>This method returns the unqualified name of the object obj's class.</p> */
   
   public static String getClassUnqualifiedName (Object obj) {
      return Strings. lastSubString (obj. getClass (). getName(), ".");
   }
   
   /** <p>This method returns the unqualified name of the class cl.</p> */
   
   public static String getUnqualifiedName (Class cl) {
      return Strings. lastSubString (cl. getName(), ".");
   }
   
   /** <p>This method finds the getter method (if any) for the specified field 
       in its declaring class.  It returns null if not found.</p>
   */
   
   public static Method findGetterMethod (Field field) {
   	
      String fieldName = field. getName ();
      String getterName = "get" + Character. toUpperCase (fieldName. charAt (0)) + fieldName. substring (1);
      Class declaringClass = field. getDeclaringClass ();
      
      try {
         Method method = declaringClass. getDeclaredMethod (getterName, parameterless);
         if (method != null 
         	 && method. getReturnType () == field. getType ()
			 && Modifier. isPublic (method. getModifiers ())) 
            return method;
         else return null;
      }
      catch (NoSuchMethodException e) { return null; }
   }
   
   /** <p>This method finds the setter method (if any) for the specified field 
       in its declaring class.  It returns null if not found.</p>
   */
   
   public static Method findSetterMethod (Field field) {
   	
      String fieldName = field. getName ();
      String setterName = "set" + Character. toUpperCase (fieldName. charAt (0)) + fieldName. substring (1);
      Class declaringClass = field. getDeclaringClass ();
      
      try {
         Method method = declaringClass. getDeclaredMethod (setterName, new Class [] {field. getType ()});
         if (method != null && Modifier. isPublic (method. getModifiers ())) 
            return method;
         else return null;
      }
      catch (NoSuchMethodException e) { return null; }
   }
   
   /** <p>This method finds all non-public fields in a given class or any of its superclasses 
       such that these fields are accessible through public getter and setter methods.</p>
   */
   
   private static Vector getNonPublicFields (Class enclosingClass) {
   	
      Vector nonPublicFields = new Vector ();
      while (enclosingClass != null) {
         Field [] field = enclosingClass. getDeclaredFields ();
         for (int ix = 0; ix < field. length; ix ++) {
            if (Modifier. isPublic (field [ix]. getModifiers ())) { } 
            else {
               if (findGetterMethod (field [ix]) != null
               	   && findSetterMethod (field [ix]) != null)
               {
                  nonPublicFields. add (field [ix]);
               }
            }
         }
         enclosingClass = enclosingClass. getSuperclass ();
      }
      return nonPublicFields;
   }
   
   /*----------------------- End Static Methods -----------------------*/

   /*------------------------- Begin Methods --------------------------*/
   
   /** <p>Returns the <b>elementary fields</b> in the given class.  A type
       is elementary if if its presentation is being managed with an ElementaryUI 
       class.  It is required that all primitive types be managed with ElementaryUI 
       classes.</p>
   */
   
   public Iterator elementaryFields (Class enclosingClass) {
   	
      Vector eFields = (Vector) elementaryFields. get (enclosingClass);
      if ( eFields == null) {
         mapAllFields (enclosingClass);
         eFields = (Vector) elementaryFields. get (enclosingClass);
      }
      return eFields. iterator ();
   }
   
   /** <p>Returns the <b>composite fields</b> for this given class.  A type  
       is composite if its presentation is not being managed with an ElementaryUI 
       class.</p>
   */
   
   public Iterator compositeFields (Class enclosingClass) {
   	
      Vector cFields = (Vector) compositeFields. get (enclosingClass);
      if ( cFields == null) {
         mapAllFields (enclosingClass);
         cFields = (Vector) compositeFields. get (enclosingClass);
      }
      return cFields. iterator ();
   }
   
   /** <p>Returns all presentable fields for this given class.</p> */
   
   public Iterator fields (Class enclosingClass) {
   	
      Vector allFields = (Vector) fields. get (enclosingClass);
      if ( allFields == null) {
         mapAllFields (enclosingClass);
         allFields = (Vector) fields. get (enclosingClass);
      }
      return allFields. iterator ();
   }
   
   /** <p>This method collects all presentable field for the given 
       class and classifies them as elementary or composite fields, 
       caching them in hash tables for subsequent retrieval.</p>
   */
   
   private void mapAllFields (Class enclosingClass) {
   	
      Vector eFields = new Vector ();
      Vector cFields = new Vector ();
      Vector selectedFields = new Vector ();
 	  Vector allFields = new Vector (); 
 	  allFields. addAll (Arrays. asList (enclosingClass. getFields ()));
 	  allFields. addAll (getNonPublicFields (enclosingClass));
 	  
 	  Iterator it = allFields. iterator ();
 	  while (it. hasNext ()) {
 	     Field field = (Field) it. next ();
 	     if (Modifier. isStatic (field. getModifiers ())) { }
		 else if (uif. hasElementaryUI (field. getType ())) {
 	        eFields. add (field);
 	        selectedFields. add (field);
		 }
 	     else {
 	     	cFields. add (field);
 	     	selectedFields. add (field);
 	     }
 	  }
 	  fields. put (enclosingClass, selectedFields);
 	  elementaryFields. put (enclosingClass, eFields);
 	  compositeFields. put (enclosingClass, cFields);
   }
   
   /*-------------------------- End Methods ---------------------------*/

   /* End of Classes */
   
}
