/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum;

import java.lang.reflect.Field;
import java.util.*;
import javax.servlet.jsp.PageContext;
import javax.servlet.http.HttpServletRequest;

import com.eknowlogie.aliorsum.prefs.Display;
import com.eknowlogie.aliorsum.prefs.Prefs;


/** <p>A CompositeUI class manages a recipient's presentation by managing the 
    presentation of its constituent fields. A composite UI relies on JSP renderers 
    to render its recipient in terms of its constituent fields.</p>
    
    @author Romel Rivera, eKnowlogie. Inc.
*/

public class CompositeUI extends UI {

   /*-------------------------- Begin Fields --------------------------*/
   
   /** <p>These are the nested UIs enclosed under this composite UI during rendering. 
       This hierarchical structure is determined during rendering through invocations to 
       beginUIRendering () and endUIRendering ().  These nested UIs may be children in 
       the recipient's object structure, or they may be unrelated objects which are being 
       rendered explicitly by a JSP renderer.</p>
       
       @see UI#enclosingUI
   */
   
   protected Set nestedUIs = new HashSet ();
   
   /** <p>This flag is true if a form was submitted by the user for this composite UI the 
       last time this UI was validated.</p>
       
       @see #wasInputSubmitted()
   */
   
   private boolean inputSubmitted = false;
   
   /** <p>This variable maintains a reference to the current elementary UI being processed 
       by this composite UI, if any.  It is a convenience variable to obtain the current 
       elementary UI being processed. Typically, this will be the value of next () when it 
       returns a UI for an elementary field during an iterator loop.  This is useful because 
       it provides access to this elementary UI for elementary JSP renderers.</p>
   */
   
   public ElementaryUI currElementaryUI = null;
   
   /** <p>If this flag is true, the titles for this UI's children are qualified with the title 
       for this UI.  This flag is used in conjunction with "flat" renderers; renderers that 
       omit the rendering of certain intermediate composite structures in order to simplify 
       the presentation and make it more readable (see "QualifiedUIRenderer.jsp").</p>
   */
   
   public boolean qualifyChildrenTitles = false;
   
   /** <p>This variable represents the page context of the current servlet request. Obviously, 
       this variable is specific to a given page (e.g. a jsp:include), therefore it needs to 
       be maintained with the page-specific component, the UI.  Maintaining it with the 
       activity, for example, would imply that it would need to be refreshed upon return from 
       a jsp:include.</p>
    
       @see #getCurrPageContext()
   */
   
   protected PageContext currPageContext = null;
   
   
   /** <p>This variable holds the table header builder suitable for a table representation 
       of this recipient.</p>
   */
   
   protected TableHeader header;
   
   /*--------------------------- End Fields ---------------------------*/

   /*----------------------- Begin Constructors -----------------------*/
   
   /** <p>This is the required empty-parameter constructor.  Instances of this class, can 
       only be created through a UI factory, which leaves these instances uninitialized.  
       </p>
   */
   
   public CompositeUI () { }
   
   /*------------------------ End Constructors ------------------------*/

   /*---------------- Begin Static Life-Cycle Methods -----------------*/
   
   /** <p>This method initiates the rendering process for a new recipient object of 
       type "recipientClass" and with rendering name "simpleName" in the current session
       "pageContext". JSP rendering of member fields in composite objects must be bracketed 
       with the beginRendering and endRendering method brackets. In a nutshell, begin rendering 
       means that the composite UI is placed in the active UI hierarchy or tree structure, and 
       set as the currently rendering UI.  If any of the {@link #presetUIRendering(Object, 
       PageContext, String) presetUIRendering} methods is used prior to the invocation of this 
       method, this composite UI can be used for non-field rendering operations (e.g. 
       {@link UI#renderTitleAndValidation() renderTitle}, {@link #getRenderer() getRenderer})  
       before this method is invoked.</p>
       
       <p>Like {@link #presetUIRendering(Object, PageContext, String) presetRendering}, this 
       method is a static factory for composite UIs and their recipients. If an active composite 
       UI for this recipient does not exist in the current activity, the local activity factory 
       (and its subordinate session factory) will use cascading preferences to select and create 
       a composite UI instance of the most appropiate subclass.  This method then invokes the 
       instance version {@link #beginUIRendering(Object, Class, PageContext, String) 
       beginUIRendering} of this static method to perform this operation.</p>
       
       @param initial     <p>Keep in mind that this method may be called repeatedly as the 
                          JSP page with the JSP renderer which calls it, goes through a 
                          validation loop. "initial" can either be the Class for the recipient 
                          object, or the non-null recipient object itself.  Passing the class 
                          is convenient when the recipient is meant to be a new object which 
                          would be created using a parameterless constructor when the composite 
                          UI is initialized the first time, otherwise we would have had to create 
                          it outside the validation loop (and outside the JSP renderer) and this 
                          way the recipient object is conveniently created only once.  If the 
                          recipient object is a pre-existing one or if it needs to be created  
                          with a parameterized constructor, then the object itself can be 
                          passed or supplied ahead of time using a presetRendering method.</p>
                             
       @param pageContext    <p>It is used to retrieve the session UI factory and the 
                             current UI activity.</p>
                             
       @param simpleName     <p>Parameter "simpleName" is used to distinguish among multiple 
                             objects of the same type being rendered in the same activity 
                             and under the same composite UI parent (e.g. an Order recipient 
                             may have a shipping and a billing address, both of type Address), 
                             otherwise this parameter is unnecessary as the recipient's type 
                             name would be used by default.</p>
                             
       @see #CompositeUI()
   */

   public static CompositeUI beginUIRendering (Object initial, PageContext pageContext, String simpleName) {
   	
   	  Class initialRecipientClass = null;
      Object initialRecipient = null;
   	  
   	  if (initial instanceof Class) 
   	     initialRecipientClass = (Class) initial;
   	  else if (initial != null) {
   	     initialRecipientClass = initial. getClass ();
   	     initialRecipient = initial;
   	  }
   	  
   	  UIActivity uia = UIActivity. getUIActivity (pageContext);
      CompositeUI ui = uia. getActiveCompositeUI (simpleName);
      if (ui == null) {
         ui = uia. localFactory. createCompositeUI (initialRecipientClass); // TODO Throw error if recipientClass == null
         ui. init (initialRecipient, initialRecipientClass, pageContext, simpleName);
      }
      return ui. beginUIRendering (initialRecipient, initialRecipientClass, pageContext, simpleName);
   }

   /** <p>Same as the fully parameterized {@link #beginUIRendering(Object,PageContext,String) 
       beginUIRendering} method, for a situation when there will be only one instance of this 
       recipient's type under the same composite UI parent. </p>
   */

   public static CompositeUI beginUIRendering (Object initial, PageContext pageContext) {
      String simpleName;
      if (initial instanceof Class)
         simpleName = Classes. getUnqualifiedName ((Class) initial);
      else simpleName = Classes. getClassUnqualifiedName (initial);
      return beginUIRendering (initial, pageContext, simpleName);
   }
   
   /** <p>Same as its equivalent but with the addition of convenience parameter "readOnly"</p>
    
       @param readOnly <p>If true, no modifications are allowed for fields of this recipient. 
                       If false, all other preferences apply. If this recipient is at the 
                       highest level of the UI hierarchy, then readOnly applies not only to 
                       this recipient but also to any other recipient with the same class in 
                       the activity.  Alternatively, this condition can be set explicitly 
                       by the client using preference registrations.</p>
   */
   
   public static CompositeUI beginUIRendering (PageContext pageContext, boolean readOnly) {
      CompositeUI ui = beginUIRendering (null, pageContext, null);
      if (readOnly) {
         if (ui. recipientField != null)
            ui. uiActivity. fieldPrefs (). register (ui. recipientField, Prefs. DISPLAY, Display. READ_ONLY); 
         else ui. uiActivity. classPrefs (). register (ui. recipientClass, Prefs. DISPLAY, Display. READ_ONLY); 
      }
      return ui;
   }
   
   /** <p>Same as the fully parameterized {@link #beginUIRendering(Object,PageContext,String) 
       beginUIRendering} method, except that it pressumes that the recipient (and possibly 
       the rendering name) have been preset with a "presetUIRendering" method.</p>
   
       @see #presetUIRendering(Object,PageContext,String) 
   */
   
   public static CompositeUI beginUIRendering (PageContext pageContext) {
      return beginUIRendering (null, pageContext, null);
   }
   
   /** <p>This method presets a composite UI for subsequent rendering and it is  
       useful in order to pre-customize the UI, for example, to assign it a specific 
       (previously-initialized) recipient, or to assign it a custom unique simple 
       name in order to distinguish it from other instances of the same UI component 
       under the same parent UI.  It will create the composite UI if one does not exist.  
       It must be followed by an invocation to a beginUIRendering method for the same 
       composite UI.  Typically the preset is invoked outside the JSP renderer for the 
       composit UI in question, while the begin rendering method is called inside the 
       actual JSP renderer. It is meant as a static factory, and it is meant to be invoked 
       before the beginRendering method that will use its presets.  Once this method has 
       been invoked, you can begin using the non field-oriented rendering functions  
       (e.g. {@link UI#renderTitleAndValidation() renderTitle}, {@link #getRenderer() getRenderer}) 
       of the returned composite UI, but you cannot use its field-oriented operations 
       (e.g. {@link #elementaryFieldUIs() elementaryFieldUIs}) until a 
       {@link #beginUIRendering(Object, Class, PageContext, String) beginUIRendering} 
       method is invoked.</p>
       
       <p>Like {@link #beginUIRendering(Object, PageContext, String) beginUIRendering}, this 
       method is a static factory for composite UIs and their recipients. If an active composite 
       UI for this recipient does not exist in the current activity, the local activity factory 
       (and its subordinate session factory) will use cascading preferences to select and create 
       a composite UI instance of the most appropiate subclass.  This method then invokes the 
       instance version ({@link #beginUIRendering(Object, Class, PageContext, String) 
       beginUIRendering}) of this static method to perform this operation.</p>
   
       @param initial     <p>Keep in mind that this method may be called repeatedly as the 
                          JSP page with the JSP renderer which calls it, goes through a 
                          validation loop. "initial" can either be the Class for the recipient 
                          object, or the non-null recipient object itself.  Passing the class 
                          is convenient when the recipient is meant to be a new object which 
                          otherwise we would have had to create outside the validation loop 
                          (and outside the JSP renderer) and this way the recipient object
                          is conveniently created only once.  If the recipient object is an
                          pre-existing one, then the object itself can be passed.</p>
                      
       @param pageContext <p>It is used to retrieve the session UI factory and the 
                          current UI activity.</p>
                             
       @param simpleName  <p>Parameter "simpleName" is used to distinguish among multiple 
                          objects of the same type being rendered in the same activity 
                          and within the same enclosing object (e.g. an Order recipient 
                          may have a shipping and a billing address, both of type Address), 
                          otherwise this parameter is unnecessary as the recipient's type 
                          name would be used by default.</p>
                          
       @see #CompositeUI()
   */
   
   public static CompositeUI presetUIRendering (Object initial, PageContext pageContext, String simpleName) {
   	  return presetUIRendering (initial, UIActivity. getUIActivity (pageContext), simpleName);
   }
   
   /** <p>Same as the fully parameterized "presetUIRendering" method, except that a 
       default simple name is created using the recipient's class name.</p>
       
       @see #presetUIRendering(Object,PageContext,String) 
   */
   
   public static CompositeUI presetUIRendering (Object initial, PageContext pageContext) {
      String simpleName;
      if (initial instanceof Class)
         simpleName = Classes. getUnqualifiedName ((Class) initial);
      else simpleName = Classes. getClassUnqualifiedName (initial);
      return presetUIRendering (initial, pageContext, simpleName);
   }
   
   /** <p>Same as its equivalent but with the addition of convenience parameter "readOnly".</p>
    
       @param readOnly <p>If true, no modifications are allowed for fields of this recipient. 
                       If false, all other preferences apply. If this recipient is at the 
                       highest level of the UI hierarchy, then readOnly applies not only to 
                       this recipient but also to any other recipient with the same class in 
                       the activity.  Alternatively, this condition can be set explicitly 
                       by the client using preference registrations.</p>
       
       @see #presetUIRendering(Object,PageContext,String) 
   */
   
   public static CompositeUI presetUIRendering (Object recipient, PageContext pageContext, boolean readOnly) {
      CompositeUI ui =  presetUIRendering (recipient, pageContext, Classes. getClassUnqualifiedName (recipient));
      if (readOnly) {
         if (ui. recipientField != null)
            ui. uiActivity. fieldPrefs (). register (ui. recipientField, Prefs. DISPLAY, Display. READ_ONLY); 
         else ui. uiActivity. classPrefs (). register (ui. recipientClass, Prefs. DISPLAY, Display. READ_ONLY); 
      }
      return ui;
   }
   
   /** <p>This is the static "funnel" to which all static presetUIRendering calls are transfered.  
       This method in turn transfers the call to the {@link #presetUIRendering(Object, Class, 
       UIActivity, String) presetUIRendering} instance method which finally implements the 
       operation in question.</p>
       
       @see #presetUIRendering(Object,PageContext,String) 
   */
   
   protected static CompositeUI presetUIRendering (Object initial, UIActivity uia, String simpleName) {
   	  
   	  Class initialRecipientClass;
      Object initialRecipient = null;
   	  
   	  if (initial instanceof Class) 
   	     initialRecipientClass = (Class) initial;
   	  else {
   	     initialRecipientClass = initial. getClass ();
   	     initialRecipient = initial;
   	  }
   	  
   	  CompositeUI ui = uia. getActiveCompositeUI (simpleName);
      if (ui == null) {
         ui = uia. localFactory. createCompositeUI (initialRecipientClass);
         ui. init (initialRecipient, initialRecipientClass, uia, simpleName);
      }
      return ui. presetUIRendering (initialRecipient, initialRecipientClass, uia, simpleName);
   }
   
   /** <p>This method recalls the CompositeUI just preset and before its begin rendering is 
       invoked.</p>
   */
   
   public static CompositeUI getPresetUI (PageContext pageContext) {
   	
   	  UIActivity uia = UIActivity. getUIActivity (pageContext);
      return uia. getActiveCompositeUI (uia. nextName);
   }

   /*----------------- End Static Life-Cycle Methods ------------------*/

   /*-------------------- Begin Life-Cycle Methods --------------------*/
   
   /** <p>This instance method is the implementation of the operations prescribed 
       by its static counterparts.  In activating this composite UI, it adds it to 
       the UI hierarchy as a child of the current UI being rendered (if not there 
       already) and it sets it as the new UI currently being rendered.  This hierarchy  
       can then be visited during recursive validation.</p>

       @see #beginUIRendering(Object, PageContext, String)
       @see #CompositeUI()
   */
   
   protected CompositeUI beginUIRendering (Object initialRecipient, Class initialRecipientClass, PageContext pageContext, String simpleName) {

   	  currPageContext = pageContext;
      uiActivity. getCurrCompositeUI (). addNestedUI (this);
      uiActivity. uiRenderingStack. push (this);
      uiActivity. clearNextName ();
      return this;
   }

   /** <p>This method is the implementation of the operations prescribed by 
       its static counterparts.  It signals that this composite UI will be 
       the one used by the next begin UI rendering operation.</p>

       @see #presetUIRendering(Object,PageContext,String)
       @see #CompositeUI()
   */
   
   protected CompositeUI presetUIRendering (Object initialRecipient, Class initialRecipientClass, UIActivity uia, String simpleName) {
      
      if (initialRecipient != null && initialRecipient != recipient) {
         recipient = initialRecipient;
      	 submissionCount = 0;
      }
      uia. setNextName (simpleName);
      return this;
   }
   
   /** <p>A convenience method for presetting a new composite UI which happens to be a 
       field in the recipient of the UI currently being rendered.</p>
       
       @see #presetUIRendering(Object,PageContext,String)
       @see #CompositeUI()
   */
   
   public CompositeUI presetUIRendering (Field field) {
      CompositeUI ui = presetUIRendering (fieldValue (field), uiActivity, field. getName ());
      ui. recipientField = field;
      return ui;
   }
   
   /** <p>This method ends the rendering for the current composite UI.  The UI is 
       no longer in the stack of composite UI's being rendered by the page but 
       it remains as an active component in the UI activity container, subject 
       to receiving input, being validated by the UI activity, and being referenced 
       to produce formatted values.</p>
   */
   
   public void endUIRendering () {
 	  uiActivity. uiRenderingStack. pop ();
   }
   
   /** <p>It initializes this class after creation by assigning it a recipient, 
       a recipient class, a rendering qualified name, and making it accessible 
       by its qualified name in the UI activity container.  It is invoked by 
       the instance beginRendering and presetRendering methods when converting 
       the UI handle to a full rendering UI.</p>
   */
   
   protected void init (Object      initialRecipient, 
   		                Class       initialRecipientClass, 
					    PageContext pageContext, 
						String      simpleName) 
   {
      init (initialRecipient, initialRecipientClass, UIActivity. getUIActivity (pageContext), simpleName);
   }
   
   /** <p>This is the implementation of {@link #init(Object, Class, PageContext, String) 
       init}</p>.
   */
   
   protected void init (Object      initialRecipient, 
   		                Class       initialRecipientClass, 
					    UIActivity  uiActivity, 
						String      simpleName) 
   {
   	  recipientClass = initialRecipientClass;
      if (initialRecipient != null) 
     	 recipient = initialRecipient;
      else {
     	 try { recipient = recipientClass. newInstance (); }
     	 catch (Exception e) { throw new Error (e. getMessage ()); }
      }
      this. uiActivity = uiActivity;
      qualifiedName = uiActivity. qualifyName (simpleName);
      uiActivity. activeUIs. put (qualifiedName, this);
   } 
   
   /** <p>This method returns the table header object to be used to render the header 
       for this composite UI.</p>
   */
   
   public TableHeader header () {
      if (header == null)
         header =  new HierarchicalTableHeader (this, recipientClass);
      return header;
   }
   
   /** <p>This method adds <code>ui</code> as a nested UI of the current composite UI being 
       rendered.  This nested UI may be a child of this recipient's object structure or it 
       may not (if it an unrelated object that was added explicitly by a JSP renderer).</p>
   */
   
   public void addNestedUI (UI ui) {
   	  ui. enclosingUI = this;
      if (! nestedUIs. contains (ui)) {
         nestedUIs. add (ui);
      }
   }
   
   /*-------------------- End Life-Cycle Methods ----------------------*/
   
   /*---------------- Begin Convenience Prefs Methods -----------------*/
   
   /** <p>Convenience method to get the Prefs.RENDERER preference for this  
       composite recipient.</p>
   */
   
   public String getRenderer () { 
      return (String) Prefs. RENDERER. get (this);
   }
   
   /*----------------- End Convenience Prefs Methods ------------------*/
   
   /*------------------- Begin Presentation Methods -------------------*/

   /** <p>Returns the unique rendering qualified name for this UI component, 
       as a prefix.  Use it to prefix field names when creating the name attribute 
       in the input tags of Html forms, during explicit rendering of elementary 
       fields.</p>
   */
   
   public String getNamePrefix () {
   	  return UIActivity. asPrefix (qualifiedName);
   }
   
   /** <p>This method returns an iterator over the presentable, filtered, fields in a 
       given target class.  Presentable fields are public fields or non-public fields 
       with getters and setters.  They are filtered with the current cascading preferences, 
       such as access == hidden etc.  They should be used for iteration by the JSP renderers 
       instead of the current UI's nestedUIs because the list of visited fields may be different 
       each time due to the effect of cascading preferences.  For example, some of the already 
       active elementary fields may not be visible in a subsequent validation pass if there 
       is a preference set to show only fields with validation errors.  The nestedUIs list by 
       contrast, contains all the recipient's field UIs that have been historically activated 
       (visited) throughout the activation.</p>
       
       @see Classes#fields(Class)
   */
   
   public Iterator fields (Class targetClass) {
      Iterator it = uiActivity. localFactory. getClasses (). fields (targetClass);
      return fieldsToVisit (it);
   }

   /** <p>This  method returns an iterator over the presentable filtered fields in the 
       recipientClass.</p>
    
       @see #fields(Class)
   */
   
   public Iterator fields () {
      return fields (recipientClass);
   }
   
   /** <p>Same as {@link #fields(Class)} except that only the elementary fields are 
       returned.</p>
       
       @see Classes#elementaryFields(Class)
   */
   
   public Iterator elementaryFields (Class targetClass) {
      Iterator it = uiActivity. localFactory. getClasses (). elementaryFields (targetClass);
      return fieldsToVisit (it);
   }
   
   /** <p>This  method returns an iterator over the presentable elementary fields in the 
       recipientClass.</p>
    
       @see #elementaryFields(Class)
   */
   
   public Iterator elementaryFields () {
      return elementaryFields (recipientClass);
   }
   
   /** <p>This method returns an iterator over the elementary UIs for the elementary 
       fields currently selected for iteration.</p>
       
       @see #elementaryFields()
   */
   
   public Iterator elementaryFieldUIs () {
      return new ElementaryUIIterator (elementaryFields (), this);
   }

   /** <p>Same as {@link #fields(Class)} except that only the composite fields are 
       returned.</p>
       
       @see Classes#compositeFields(Class)
   */
   
   public Iterator compositeFields (Class targetClass) {
      Iterator it = uiActivity. localFactory. getClasses (). compositeFields (targetClass);
      return fieldsToVisit (it);
   }
   
   /** <p>This  method returns an iterator over the presentable composite fields in the 
       recipientClass.</p>
    
       @see #compositeFields(Class)
   */
   
   public Iterator compositeFields () {
      Iterator it = uiActivity. localFactory. getClasses (). compositeFields (recipientClass);
      return fieldsToVisit (it);
   }
   
   /** <p>This method returns an iterator with the composite UIs for the composite 
       fields currently selected for iteration.  This iterator conveniently presets 
       each composite UI before it is returned by next (), so that it can be used 
       for rendering non-composite properties or be ready for the begin rendering of 
       nested JSP renderers.</p>
       
       @see #compositeFields()
   */
   
   public Iterator compositeFieldUIs () {
      return new CompositeUIIterator (compositeFields (), this);
   }
   
   /** <p>This method returns an iterator with the UIs for the children in the recipient's
       object structure which were selected for iteration.  This iterator conveniently  
       presets each composite UI before it is returned by next (), so that they can be used 
       for rendering non-composite properties right away or be ready for the begin rendering  
       in nested JSP renderers.  Notice that nested UIs which are not part of this recipient's 
       object structure are not included in this iteration.  This method returns all the 
       selected fields in the recipient's object structure, but this method can be overriden 
       to return the list elements in a list recipient, or array elements in an array 
       recipient.</p>
       
       @see #compositeFields()
   */
   
   public Iterator childrenUIs () {
      return new UIIterator (fields (), this);
   }
   
   /** <p>This class is an iterator over the UIs for this recipient's fields.  If the field
       has a preferenced elementary UI class, then an elementary UI instance for this field 
       is returned, otherwise an already preset composite UI instance is returned for this 
       field.</p>
   */
   
   public class UIIterator implements Iterator {
   	
      CompositeUI parentUI;
   	  Iterator it;
   	  
      public UIIterator (Iterator it, CompositeUI parentUI) { 
         this. it = it; 
      	 this. parentUI = parentUI;
      }
      
      public boolean hasNext () { 
         return it. hasNext (); 
      }
      
      /** <p>If the field has a preferenced elementary UI class, then an elementary UI 
          instance for this field is returned, otherwise an already preset composite UI 
          instance is returned for this field.</p>
      */
      
      public Object next () {
         Field field = (Field) it. next ();
         currElementaryUI = null;
         if (uiActivity. localFactory. hasElementaryUI (field. getType ())) {
            currElementaryUI = uiActivity. activeElementaryUI (field, parentUI);
            return currElementaryUI;
         }
         else return parentUI. presetUIRendering (field);
      }
      
      public void remove () { it. remove (); }
   }
   
   /** <p>This class is an iterator over the composite UIs for this recipient's fields, 
       and is used to preset for rendering each composite UI prior to delivery by next ().</p>
   */
   
   public class CompositeUIIterator extends UIIterator {
   	  
      public CompositeUIIterator (Iterator it, CompositeUI parentUI) { 
         super (it, parentUI);
      }
      
      /** <p>Presets for rendering the composite UI of this recipient's field before 
          returning it as next ().</p>
      */
      
      public Object next () {
         Field cField = (Field) it. next ();
         return parentUI. presetUIRendering (cField);
      }
      
   }
   
   /** <p>This class is an iterator over the elementary UIs for this recipient's elementary 
       fields.</p>
   */
   
   public class ElementaryUIIterator extends UIIterator {
   	  
      public ElementaryUIIterator (Iterator it, CompositeUI parentUI) { 
         super (it, parentUI);
      }
      
      /** <p>Returns the elementary UI for the next field.</p> */
      
      public Object next () {
         Field eField = (Field) it. next ();
         currElementaryUI = uiActivity. activeElementaryUI (eField, parentUI);
         return currElementaryUI;
      }
      
   }
   
   /** <p>This method returns the elementary UI for this field name.  This is 
       convenient when a JSP renderer is not using the elementary field UI iterator 
       for rendering ({@link #elementaryFieldUIs()}).</p> 
   */
   
   public ElementaryUI elementaryFieldUI (String fieldName) {
      currElementaryUI = uiActivity. activeElementaryUI (Fields. getField (fieldName, recipientClass), this);
      return currElementaryUI;
   }
   
   /** <p>This method returns the elementary UI for this elementary recipient.  This is 
       convenient when this recipient is not a field in the enclosing UI, perhaps it is 
       an object not part of the domain model and just used temporarily during the 
       presentation.  Its simple name must be unique within the enclosing UI.</p> 
       
       <p>If this recipient object is modified during the input and validation loop, 
       the entire object might be replaced which means that any external references to 
       the recipient object would need to be updated.  Some elementary objects may 
       be updated by modifying its internal state without requiring the entire object 
       to be replaced.  See the elementary UIs for specific objects.</p>
       
       @see ElementaryUI#setValue(Object)
   */
   
   public ElementaryUI elementaryUI (Object recipient, String simpleName) {
      currElementaryUI = uiActivity. activeElementaryUI (recipient, simpleName, this);
      return currElementaryUI;
   }
   
   /** <p>This method returns the elementary UI for this elementary recipient.  This is 
       convenient when this recipient is not a field in the enclosing UI, perhaps it is 
       an object not part of the domain model and just used temporarily during the 
       presentation.  Its simple name is its class name and assumed to be unique within 
       the enclosing UI.</p> 
       
       @see #elementaryUI(Object, String)
   */
   
   public ElementaryUI elementaryUI (Object recipient) {
      return elementaryUI (recipient, Classes. getClassUnqualifiedName (recipient));
   }
   
   /** <p>This method returns the composite UI for this field name.  This is 
       convenient when a JSP renderer is not using the composite field UI iterator 
       for rendering ({@link #compositeFieldUIs()}).</p> 
   */
   
   public CompositeUI getCompositeUI (String fieldName) {
      currElementaryUI = null;
      Field cField = Fields. getField (fieldName, recipientClass);
      return presetUIRendering (cField);
   }
   
   /** <p>Returns true if a form was submitted for this composite UI.  For each 
       set of inputs in a form, corresponding to a given recipient handled by a 
       composite UI, there has to be a standard hidden form input field with a 
       name of prefix + "formSubmitted" and a value of "yes", which will serve 
       as an indicator that the values for this UI component have been submitted.</p>
   */
   
   public boolean isFormSubmitted (PageContext pageContext) {
      if (this == uiActivity. root)
         return true;
      else {
      	 HttpServletRequest request = (HttpServletRequest) pageContext. getRequest ();
         String formSubmitted = request. getParameter (getNamePrefix () + "formSubmitted");
         return formSubmitted != null && formSubmitted. equalsIgnoreCase ("yes");
      }
   }
   
   /** {@inheritDoc}
       
       <p>Returns true if a form for this composite UI was submitted the last time a form 
       for this activity was submitted.</p>
       
       @see ElementaryUI#wasInputSubmitted()
   */
   
   public boolean wasInputSubmitted () {
      return inputSubmitted;
   }
   
   /** {@inheritDoc}
    
       <p>This method validates this composite recipient by recursively validating 
       its nested UIs.  If they all were valid, then it validates the integrity of 
       this composite recipient by validating its field inter-relations, as these 
       fields have been modified by user input.  If this composite recipient's 
       integrity was not valid and no corrective adjustments were possible, one or 
       more validation messages are posted with this composite UI and false is 
       returned.</p>
   */
   
   public boolean validate (PageContext pageContext) {

   	  inputSubmitted = isFormSubmitted (pageContext);
      if (! inputSubmitted) {
         clearInputSubmitted (this);
         return true;
      }
      
      valid = true;
      Iterator it = nestedUIs. iterator ();
      while (it. hasNext ()) {
         valid = ((UI) it. next ()). validate (pageContext) && valid;
      }
      valid =  validateComposite () && valid;
      return valid;
   }
   
   /** <p>This method raises an exception since it is not implemented for this base CompositeUI 
       class.  You need to use a JSP renderer in order to render this recipient by rendering 
       its constituent fields.</p> 
   */
   
   public String render () {
     throw new Error ("Render operation not implemented for the base CompositeUI class.");
   }
   
    /** <p>This method raises an exception since it is not implemented for this base CompositeUI 
       class.  You need to use a JSP renderer in order to render this recipient by rendering 
       its constituent fields.</p> 
   */
    
   public String renderVal () {
     throw new Error ("Render operation not implemented for the base CompositeUI class.");
   }
   
   /** <p>Validates the recipient itself as a group.  It is used only once all 
       the fields for this recipient have passed validation.  This is used to 
       validate relationships between fields, etc. </p>
   */
   
   protected boolean validateComposite () { 
   	  validationMessages = new HashSet ();
   	  if (recipient instanceof Validator) {
   	     try { ((Validator) recipient). validate (); }
   	     catch (InvalidState e) {
   	     	String msg = e. getMessage ();
   	     	if (msg != null && msg. trim () != "")
   	           addValidation (e. getMessage ());
   	     	Iterator it = e. fieldMessages. entrySet (). iterator ();
   	     	while (it. hasNext ()) {
   	           Map. Entry entry = (Map. Entry) it. next ();
   	           String fieldName = (String) entry. getKey ();
   	           String qualifiedFieldName = UIActivity. asPrefix (qualifiedName) + fieldName;
               UI ui = (UI) uiActivity. activeUIs. get (qualifiedFieldName);
               ui. addValidation ((String) entry. getValue ());
               ui. valid = false;
   	     	}
   	     	valid = false;
   	        return false;
   	     }
   	  }
      return true; 
   }

   /** <p>Convenience method to retrieve the value of a field of recipient's. 
       if the value is null, the field is set to a new instance.</p> 
       
       @see Fields#fieldValue(Field, Object)
   */
   
   public Object fieldValue (Field field) {
      return Fields. fieldValue (field, recipient);
   }
   
   /** <p>This method returns the page context for the current servlet request.</p> */
   
   public PageContext getCurrPageContext () {
      return currPageContext;
   }
   
   /** <p>Convenience (internal) method to retrieve a public field (or a non-public 
       field with a public getter and setter) member of the recipient's class.</p>
       
       @see Fields#getField(String, Class)
   */
   
   public Field getField (String fieldName) {
      return Fields. getField (fieldName, recipientClass);
   }
   
   /** <p>This method returns the fields to be visited by this UI by applying inclusion / 
       exclusion preferences or hand-coded criteria to a list of presentable fields.</p>
   */
   
   protected Iterator fieldsToVisit (Iterator it) {
      Vector filteredFields = new Vector ();
      while (it. hasNext ()) {
         Field field = (Field) it. next ();
         if (willVisitField (field))
         	filteredFields. add (field);
      }
      return filteredFields. iterator ();
   }
   
   /** <p>This method returns true if this field is to be visited by this UI by applying 
       inclusion / exclusion preferences or hand-coded criteria.  Override this method 
       to create your own field selection mechanisms.</p>
   */
   
   protected boolean willVisitField (Field field) {
      // TODO this should return the active UI for this field.
      UI ui = uiActivity. getActiveUI (field. getName ());
      
      boolean willVisit = true;
      if (Prefs. ACCESS. isHidden (field, this)) 
         willVisit = false;
      else {
         willVisit = ! (wasInputSubmitted () 
         		        && Prefs. WHEN_INVALID. isTrue (field, this) && (ui == null || ui. isValid ())); 
      }
      clearInputRequested (ui);
      return willVisit;
   }
   
   /** <p>This method indicates that the user did not submit input in this validation 
       iteration for this composite "ui" and its descendants and therefore they will not 
       be validated.  It clears their validation progress from the previous validation 
       iteration.  It resets the flag {@link #inputSubmitted} for "ui" and its composite 
       nestedUIs.  If {@link UIActivity#displayValidationAfterHiatus displayValidationAfterHiatus} 
       is false, it also clears the validation state of its elementary descendants.
   */
   
   private void clearInputSubmitted (CompositeUI ui) {
      ui. inputSubmitted = false;
      Iterator it = ui. nestedUIs. iterator ();
      while (it. hasNext ()) {
         UI child = (UI) it. next ();
         if (child instanceof CompositeUI)
         	clearInputSubmitted ((CompositeUI) child);
         else if (! uiActivity. displayValidationAfterHiatus) {
         	((ElementaryUI) child). clearValidation ();
         }
      }
   }

   
   /** <p>This method indicates that this field will not be visited in this rendering 
       iteration and therefore neither it nor its elementary descendants, if any, will 
       be requesting input during this iteration.</p>
   */
   
   private void clearInputRequested (UI ui) {
      if (ui instanceof ElementaryUI)
         ((ElementaryUI) ui). inputRequested = false;
      else if (ui instanceof CompositeUI) {
      	Iterator it = ((CompositeUI ) ui). nestedUIs. iterator ();
      	while (it. hasNext ()) {
      		clearInputRequested ((UI) it. next ());
      	}
      }
   }

   /*-------------------- End presentation Methods ---------------------*/

   /* End of CompositeUI */
}