/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum;

import java.util.HashSet;
import java.lang.reflect.Field;
import javax.servlet.jsp.PageContext;
import javax.servlet.http.HttpServletRequest;

import com.eknowlogie.aliorsum.prefs.Prefs;


/** <p>An elementary UI class is used to manage a recipient (including all 
    primitive types) as one single indivisible value. Like CompositeUI classes, 
    ElementaryUI classes also hold internally a reference to the recipient 
    they process.  For simplicity, and contrary to composite UIs, they dont 
    rely on external JSP renderers, but rather, they provide internal rendering 
    through self-contained rendering methods which can be overriden to create 
    arbitrary rendering for these elementary objects.</p>
    
    @author Romel Rivera, eKnowlogie. Inc.
*/

public class ElementaryUI extends UI {
	
   /** <p>This flag is true if this elementary UI requested input to the user during 
       the last rendering of its enclosing composite UI (i.e. as an input field in an 
       Html form).</p>
       
       @see #wasInputSubmitted()
   */
   
   protected boolean inputRequested = false; 
   
	
   /*----------------------- Begin Constructors -----------------------*/
   
   /** <p>This is the required empty-parameter constructor.  Instances of this class  
       can only be created through a UI factory, but contrary to composite UIs, they 
       are initialized right away.</p>
   */
	
   public ElementaryUI () { }
   
   /*------------------------ End Constructors ------------------------*/

   /*------------------------- Begin Methods --------------------------*/
	
   /** <p>It initializes this class after creation by assigning it the recipient 
       field and the parent UI.  With those two internal variables the actual 
       recipient object can be read and set.  This method also assigns it a 
       unique qualified rendering name and activates this UI as a child of the 
       enclosing composite UI.</p>
   */
   
   public void init (Field field, CompositeUI parentUI) {
      recipientField = field;
	  uiActivity = parentUI. uiActivity;
      qualifiedName = uiActivity. qualifyName (field. getName ());
      uiActivity. activeUIs. put (qualifiedName, this);
      parentUI. addNestedUI (this);
   }
   
   /** <p>Equivalent to {@link #init(Field, CompositeUI)} except that this is not a field 
       in the enclosing UI, and therefore the recipient object is initialized instead of 
       the recipient field.</p>
   */
   
   public void init (Object recipient, String simpleName, CompositeUI parentUI) {
      this. recipient = recipient;
	  uiActivity = parentUI. uiActivity;
      qualifiedName = uiActivity. qualifyName (simpleName);
      uiActivity. activeUIs. put (qualifiedName, this);
      parentUI. addNestedUI (this);
   }
	
   /** {@inheritDoc}
       
       <p>This method validates a single input value for this elementary recipient 
       according to its type.  This is the public, client-accessible method of the 
       so called validation methods used to easily implement and subclass the validation 
       process.  This method relies on separate methods {@link #validate(String)} to 
       set the recipient's value, {@link #parse(String)} to parse, convert the input and 
       post validation messages, and {@link #isMissingRequired(String)}.  Override any 
       of these methods when subclassing an elementary UI.</p>
   */
	
   public boolean validate (PageContext pageContext) {
   	      
      HttpServletRequest request = (HttpServletRequest) pageContext. getRequest ();
   	  String inputValue = request. getParameter (qualifiedName);
   	  
   	  valid = true;
   	  if (isReadOnly () || ! wasInputSubmitted ()) 
   	     return true;
   	  
   	  invalidInputValue = null;
   	  validationMessages = new HashSet ();
   	  
   	  if (isMissingRequired (inputValue)) 
   	     addValidation (MISSING_REQUIRED);
   	  else {
   	     submissionCount ++;
   	     if (validate (inputValue)) 
		    return true;
   	     else invalidInputValue = inputValue;
	  }
   	  valid = false;
   	  return false;
   }
   
   /** <p>This method returns true if the input value corresponds to 
       missing input (e.g. null, blank, etc.) and this recipient is required 
       to have a value.  It is used by {@link #validate(PageContext)}.  Override 
       when your recipient types have different definitions of missing (e.g. 
       a boolean as checkbox is never missing, its either true or false).</p>
   */
    
   protected boolean isMissingRequired (String inputValue) {
      return (inputValue == null || inputValue. trim (). equals ("")) 
       	     && Prefs. IS_REQUIRED. isTrue (this);
   }
    
   /** <p>This method is used by {@link #validate(PageContext)}.  This method sets 
       the recipient to a non-null value received from {@link #parse(String)}.  This 
       method is meant only for internal use by the validation methods, specially 
       when subclassing this class. 
       </p> 
   */
    
   protected boolean validate (String inputValue) {
      Object value = parse (inputValue);
      setValue (value);
	  return value != null;
   }
   
   /** <p>This method sets the value of this elementarty recipient to <code>value</code>. As  
       we know, the recipient may be represented as a field or an object.  If this elementary 
       recipient is maintained as an object and not as a field, this method replaces the 
       recipient object with <code>value</code>, and thus any references to the old recipient 
       in the presentation or client code would need to be updated as shown below.</p>
       
       <pre>
       String userName = "Juan";
       ElementaryUI userNameUI = ui. elementaryUI (userName);
       ... setValue ("Pedro");... // called implicitly during the validation loop 
       userName = (String) userNameUI. getRecipient ();
       </pre>
       
       For some elementary recipient objects, this method may be overriden in such a way 
       that the recipient object itself will not be replaced, instead only its internal
       state modified.</p>
   */
   
   protected void setValue (Object value) {
      if (value != null && recipientField != null)
         Fields. setFieldValue (recipientField, enclosingUI. getRecipient (), value);
      else if (value != null) 
         recipient = value;
   }
   
   /** <p>This method parses the input value and either converts the input into 
       an object of the recipient's type, or it posts one or more validation messages 
       with this UI.  It does not set the recipient to the resulting converted value 
       as that is done by {@link #validate(String)}.  This method is meant only 
       for internal use by the validation methods, specially when subclassing this 
       class. </p>
   */
    
   protected Object parse (String inputValue) {
      return inputValue;
   }
    
   /** <p>Returns the invalid input from last submission for this recipient, 
       if any, null otherwise.</p> 
   */
   
   public String getInvalidInput () {
      return invalidInputValue;
   }
   
   /** <p>If the last user input value for this recipient's field was invalid, 
       then that invalid text is returned by this method, otherwise this method 
       formats for output the value of the recipient.  The superclass method 
       formats the recipient's value with "toString ()" which may be appropiate 
       for many subclasses.</p>
   */

   public String format () {

      if (invalidInputValue != null) 
         return invalidInputValue;
      else {
         Object value = getValue ();
         if (value == null)
            return "";
         else return value. toString ();
      }
   }
   
   /** <p>This method returns the value of the recipient.</p> */
   
   protected Object getValue () {
      if (recipientField != null)
         return Fields. getFieldValue (recipientField, enclosingUI. getRecipient ());
      else return recipient;
   }
    
   /** <p>This method renders this elementary recipient according to the Prefs. ACCESS 
       preference applicable to this recipient (e.g. it renders this field for read only, 
       write only, or read and write access modes).  It displays the field title, validation 
       messages if appropriate, and the input field initialized to its last valid or invalid 
       value, as appropiate.</p>
   */
   
   public String render () {
      return renderTitleAndValidation () + renderVal ();
   }
    
   /** <p>Similar to {@link #render()}, except that only the value of the recipient is 
       displayed, without a title and without validation messages.</p>
   */
    
   public String renderVal () {
      if (isReadOnly ()) { 
   	     return format ();
   	  }
      else {
      	 inputRequested = true;
         return "<input type = \"text\" name = \"" + qualifiedName + "\" " + 
                " value = \"" + format () + "\" /> ";
      }
   }
    
   /** <p>Similar to {@link #render()}, except that the title and value are displayed 
       as a table row with two columns.</p>
   */
    
   public String renderCols () {
      return "<tr><td>" + renderTitleAndValidation () + "</td><td>" + renderVal () + "</td></tr>";
   }
   
   /** {@inheritDoc}
       
       <p>Returns true if this elementary UI was rendered for input and its enclosing composite 
       UI was submitted an input form from the user the last time a form for this activity was 
       submitted.  Notice that the user may still have decided to provide an input value = null 
       for this elementary recipient.  We differentiate between submitting an input with a null 
       value (when submitting a form which includes a visible field not filled in) and not 
       submitting input (because the form containing this field was not submitted or this field 
       was not rendered for input in a form that was submitted).</p>
   */
   
   public boolean wasInputSubmitted () {
      return inputRequested && enclosingUI. wasInputSubmitted ();
   }
   
   /** <p>This method clears the results of a validation.  This is useful when we do not want 
       to display validation information during a subsequent rendering.  This also pressumes 
       that this elementary field will be rendered and validated again so a lingering invalid 
       value would get recognized.</p>
   */
   
   public void clearValidation () {
      inputRequested = false;
      validationMessages = null;
      valid = true;
   }
    
   /*-------------------------- End Methods ---------------------------*/

   /* End of ElementaryUI */
}
