/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum;

import java.lang.reflect.*;

/** <p>This class provides static reflective methods for getting a field, 
    and getting and setting the value of a field, where the field may be either 
    public or else non-public with public getters and setters, and declared 
    in a given class or any of its superclasses.  These operations support the 
    Field manipulation functionality needed by Gaviidae, and all such 
    field manipulation should always be centralized through these methods.</p> 
    
    @author Romel Rivera, eKnowlogie. Inc.
*/

public class Fields {
	
   /** <p>This method finds and returns the field with name "fieldName" in 
       class "enclosingClass", where the field may be either public or 
       else non-public but with public getters and setters and declared in 
       the given enclosing class or any of its superclasses.</p>
   */
	
   public static Field getField (String fieldName, Class enclosingClass) {
      try { 
         Field field = enclosingClass. getField (fieldName); 
         if (field != null) {
            return field;
         }
         while (enclosingClass != null) {
            field = enclosingClass. getDeclaredField (fieldName); 
            if (field != null) {
               return field;
            }
            else enclosingClass = enclosingClass. getSuperclass ();
         }
         return null;
      }
      catch (NoSuchFieldException e) { throw new Error (e. getMessage ()); }
   }
   
   /** <p>This method returns the value of a field in a given enclosing
       object.  If the field is non-public, then a getter method for the 
       field is assumed and used to retrieve the value.  If the value is 
       null then it sets the value of that field to a new instance.  We 
       do not use the notation "getFieldValue" to indicate that this method 
       is both a getter and setter.</p>
   */
   
   public static Object fieldValue (Field field, Object enclosingObject) {
      Object value = getFieldValue (field, enclosingObject);
      if (value == null) {
         try {
            value = field. getType (). newInstance ();
            setFieldValue (field, enclosingObject, value);
         }
         catch (Exception e) { throw new Error (e. getMessage ()); }
      }
      return value;
   }
   
   /** <p>This method returns the value of a field in a given enclosing
       object.  If the field is non-public, then a getter method for the 
       field is assumed and used to retrieve the value.</p>
   */

   public static Object getFieldValue (Field field, Object enclosingObject) {
      try { 
         if (Modifier. isPublic (field. getModifiers ())) { 
            return field. get (enclosingObject); 
         }
         else {
            Method getterMethod = Classes. findGetterMethod (field);
            return getterMethod. invoke (enclosingObject, new Object [] {});
         }
      }
      catch (Exception e) { throw new Error (e. getMessage ()); }
   }
   
   /** <p>This method sets the value of a field in a given enclosing
       object.  If the field is non-public, then a setter method for the 
       field is assumed and used to set the value.</p>
   */
   
   public static void setFieldValue (Field field, Object enclosingObject, Object value) {
      try { 
         if (Modifier. isPublic (field. getModifiers ())) { 
            field. set (enclosingObject, value); 
         }
         else {
            Method setterMethod = Classes. findSetterMethod (field);
            setterMethod. invoke (enclosingObject, new Object [] {value});
         }
      }
      catch (Exception e) { throw new Error (e. getMessage ()); }

   }
   
  /* End of Fields */
}
