/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum; 

import java.util.*;
import java.lang.reflect.Field;

/** <p>This class is used to render a table header for a table which displays an object's 
    value by displaying all the elementary fields in its structure.    The entire object 
    tree structure composition is represented hierarchically in the table header. There are 
    as many rows in the header as levels in the tree structure.  Every composite field in 
    the object tree is represented with a header spanning over the subheaders of all of its 
    fields.  The header contains as many columns as there are displayable elementary fields 
    in the object structure.</p>
    
    @author Romel Rivera, eKnowlogie. Inc.
*/

public class HierarchicalTableHeader extends TableHeader {
   	
   /** <p>These are the number of levels in the object's tree structure.</p> */
	
   private int depth = 0;
      
   /** <p>Temporary variable used by methods in this class to produce the rendering 
       of this header.</p>
   */
      
   private int currDepth = 0;
      
   /** <p>This is a list of levels in the object tree, where each fieldsByLevel contains 
       the list of all fields for that fieldsByLevel in the tree.</p>
   */
      
   private Vector fieldsByLevel;
      
   /** <p>This hash table contains the number of presentable elementary fields directly or 
       indirectly nested in each class which is a field type in this object structure. The 
       map entry is (Class, Integer).</p>
   */
      
   private Hashtable numberOfElementaryFields = new Hashtable ();
      
   /** <p>This is the set of all the elementary classes used in the object structure.</p> */
      
   private Set leaves;
   
   private int numCols = 0;
      
   /** <p>This constructor indicates that this table header will be used for rendering 
       of the table through this composite UI, and for an object of type headerClass.</p>
   */
      
   public HierarchicalTableHeader (CompositeUI ui, Class headerClass) {
      super (ui, headerClass);
   }
      
   public String render () {
      	
      if (cached == true && header != null)
         return header;
         
      buildHeader ();
      header = "<thead class = \"listHeader\">";
      int currDepth = 0;
      Iterator rows = fieldsByLevel. iterator ();
      while (rows. hasNext ()) {
         currDepth ++;
         Iterator columns = ((Vector) rows. next ()). iterator ();
         header += "<tr>";
         while (columns. hasNext ()) {
            Object column = columns. next ();
            if (column instanceof Field) {
               Field columnField = (Field) column;
               header += "<th ";
               int rowSpan = depth - currDepth;
               if (leaves. contains (columnField. getType ()) && rowSpan > 1) {
                  header += "rowspan = \"" + rowSpan + "\" ";
               }
               int colSpan = ((Integer) numberOfElementaryFields. get (columnField. getType ())). intValue ();
               if (colSpan > 1) 
                  header += "colspan = \"" + colSpan + "\" ";
               header += ">" + ui. renderTitle (columnField) + "</th>";
            }
            else if (column instanceof String) {
               header += "<th ";
               int rowSpan = depth - currDepth;
               if (rowSpan > 1) {
                  header += "rowspan = \"" + rowSpan + "\" ";
               }
               header += ">" + column + "</th>";
            }
         }
         header += "</tr>";
      }
      header += "</thead>";
      return header;
   }
   
   public void buildHeader () {
      if (fieldsByLevel == null) {
         fieldsByLevel = new Vector ();
         leaves = new HashSet ();
         if (headerClass == null) {
            numCols = 0;
            fieldsByLevel. add (new Vector ());
         }
         else numCols = getNumberOfElementaryFields (headerClass);
      }
   }
   
   public void addColumn (String colTitle) {
      buildHeader ();
      header = null;
      ((Vector) fieldsByLevel. get (0)). add (colTitle);
      numCols ++;
   }
   
   public int numCols () {
      return numCols;
   }
   
   /** <p>This method analyzes the class tree structure for <code>thisClass</code>.  It 
       calculates 1) the number of elementary fields directly or indirectly nested in each  
       class in the tree, 2) the number of levels in the tree, 3) the list of fields for 
       each level in the tree, and 4) the set of leaves in the tree.  All such values get 
       stored in local variables in this class.</p>
   */
   
   private int getNumberOfElementaryFields (Class thisClass) {
   	
      if (++ currDepth > depth)
         depth = currDepth;
      
      if (ui. uiActivity. localFactory. hasElementaryUI (thisClass)) {
         leaves. add (thisClass);
         numberOfElementaryFields. put (thisClass, new Integer (1));
         currDepth --;
         return 1;
      }
         
      int thisNumberOfElementaryFields = 0;
      Iterator it = ui. fields (thisClass);
      while (it. hasNext ()) {
         Field field = (Field) it. next ();
   	     if (fieldsByLevel. size () < currDepth)
   	        fieldsByLevel. add (new Vector ());
   	     ((Vector) fieldsByLevel. get (currDepth - 1)). add (field);
         thisNumberOfElementaryFields += getNumberOfElementaryFields (field. getType ());
      }
      numberOfElementaryFields. put (thisClass, new Integer (thisNumberOfElementaryFields));
      currDepth --;
      return thisNumberOfElementaryFields;
   }
   
   /* End of HierarchicalTableHeader */
}

   