/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum;

import java.util.*;
import java.lang.reflect.Field;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.BodyContent;

import com.eknowlogie.aliorsum.prefs.Prefs;

/** <p>A UI class manages the presentation of an object called its recipient,  
    to which it keeps a reference in its local state.  A UI provides the following 
    presentation functions: input, validation, formatting and rendering (presentation 
    control is managed by a UI activity).  It makes use of cascading preferences  
    available from the activity and the session factories in order to customize 
    these functions.  UI objects are dedicated to their recipients; that is, the 
    recipient for a given UI cannot be changed once initialized.  UI objects are 
    created by the UI factory and that requires that all UI subclasses provide an 
    empty parameter constructor for creation, supported with subsequent initialization 
    methods.</p>
    
    <p>A JSP page can be used as a <b>JSP Renderer</b> for a UI.  A JSP renderer is 
    <b>external</b> if it is invoked externally by another JSP page through a jsp:include 
    tag.  A JSP renderer is <b>internal</b> if it is invoked by a rendereing method through 
    a server-side include operation ({@link #include(String)}), in which case the rendering 
    method receives the JSP page output into a string when preparing its own rendering 
    output.  For an example of the use of an internal renderer, see the body of 
    <code>CounterUI.renderVal()</code>. An internal renderer (i.e. one invoked programatically 
    through a server-side include, must be bracketed by {@link #beginOutputCollection(PageContext)} 
    and {@link #endOutputCollection()} so that its output stream may be collected by its 
    caller.</p>
    
    <p><b>JSP Renderers for Composite UIs</b>.  JSP pages used to render composite 
    UIs must be bracketed with {@link CompositeUI#beginUIRendering(Object, PageContext, 
    String) beginUIRendering} and {@link CompositeUI#endUIRendering()} operations and must    
    have all of their elementary field or element rendering operations in the same page.  
    The reason for this is that elementary UI renderers may make use of their own JSP 
    renderers, which are currently invoked from the composite UI's page context
    (see {@link UI#include(String)}).  Since read and write composite JSP renderers are 
    used to process forms, they are required to contain a hidden input field with a qualified 
    simple name <code>formSubmitted</code> because this will serve as a flag to indicate that 
    input was submitted for the composite UI rendered by this page.</p>
    
    <p>An <b>activity page</p> is a JSP page which includes the outermost JSP renderers and 
    where the activity validation takes place.  The form tags should always be defined in 
    an activity page, and an unqualified <code>formSubmitted</code> hidden input must also be 
    defined.  In the form tag's action, control is always assumed to be given to the same 
    page without any of the previous page-control query parameters, ans so that control can 
    be managed by the activity instead, thus <code>action = "?"</code> is appropiate for this 
    purpose.</p>
    
    <p><b>JSP Renderers for Elementary UIs</b>.  JSP pages used to render elementary 
    UIs must always be internal renderers, since they always need to be invoked through 
    rendering methods.  Contrary to composite JSP renderers, elementary renderers do 
    not use <code>beginUIRendering</code> and <code>endUIRendering</code> brackets.  
    See the provided <code>CounterUIRenderer.jsp</code> for an example.</p>
    
    
    @author Romel Rivera, eKnowlogie. Inc.
*/

public abstract class UI {

   /*-------------------------- Begin Fields --------------------------*/
	
   /** <p>This recipient is the domain model object whose presentation is being  
       managed by this CompositeUI instance.  If composite, its field values (along 
       with invalid input values) are displayed during iterative validation, or 
       during presentation initialization.</p> 
   */
   
   protected Object recipient;

   /** <p>If the recipient object is being processed as a field in an enclosing 
       recipient, this variable denotes that field.  Having this optional variable 
       set is valuable because it allows cascading preference selection to start with 
       the more specific field preferences.  It can be used by this UI to create a 
       recipient object on the fly, if one does not exists, provided the field type 
       has an empty parameter constructor (which is the same restriction for model 
       object persistence in Hibernate).  Create the recipient in advance if it needs
       to be pre-initialized or if it requires a non-empty parameter constructor.</p>
   */
	
   public Field recipientField;
	
   /** <p>The class of the recipient.  If <code>recipientField</code> is null, then 
       this variable must be non-null, to insure that we can create the recipient on 
       the fly and/or the properly cascade through field or class preferences. It can 
       be used by this UI to create a recipient object on the fly, if one does not 
       exists, provided this <code>recipientClass</code> has an empty parameter 
       constructor (which is the same restriction for model object persistence in 
       Hibernate).  Create the recipient in advance if it needs to be pre-initialized 
       or if it requires a non-empty parameter constructor.</p> 
   */
   
   public Class recipientClass;
	
   /** <p>If an input value is valid, it is converted and used to set the corresponding 
       recipient object.  If the value is invalid, it is kept in this variable where it 
       can be retrieved for redisplay.</p> 
   */
   
   protected String invalidInputValue = null;
   
   /** <p>Any messages that need to be presented to the user are stored in this 
       variable as a collection.  These messages do not have to be all remedial 
       validation messages, they can also be preventive, initial explanatory  
       messages describing input format rules, etc.  These messages are cleared 
       when a new input submission for this recipient is received from the user.  
       More than one message can be posted in this collection.</p>
   */
   
   protected Set validationMessages = new HashSet ();
   
   /** <p>Counts the number of times input for this recipient is submitted by 
       the user during an activity.</p> 
   */
   
   protected int submissionCount = 0;
   
   /** <p>This flag is false only when the last input for this recipient was not 
       valid.</p>
   */
   
   protected boolean valid = true;
   
   /** <p>This is the rendering name for this UI.  That is, the name that 
       identifies this UI uniquely in Html forms during rendering.</p>
   */
   
   protected String qualifiedName;
   
   /** <p>This is the parent UI to which this UI belongs as a child.  The recipient's 
       object tree structure will be replicated as an UI tree structure during presentation. 
       Notice that the developer can produce an UI tree different from the object tree. 
       Examples: 1) by hiding selected fields, 2) by rendering an unrelated object inside 
       the rendering of another object, and 3) by flattening the object tree, hiding 
       deeply-nested objects and rendering them at an outer or the outermost level in 
       the activity.</p>
   */
   
   protected CompositeUI enclosingUI = null;
   
   /** <p>This is the activity container to which this UI belongs during rendering.  
       The life-time of a UI is intended to be the same as that of the activity it 
       belongs to.  A UI would cease to exist once its recipient has been fully saved 
       and once the activity has been ended.</p>
   */
   
   protected UIActivity uiActivity;
   
   protected boolean out;
   
   /** <p>The validation message for missing required fields.</p> */
   
   protected static final String MISSING_REQUIRED = "This field is required.";
   
   /** <p>Generic validation message.</p> */
	
   protected static final String INVALID_FORMAT = "Invalid input.";
   
   /** <p>Numeric validation message.</p> */
   
   protected static final String INVALID_NUMERIC_FORMAT = "This field must be numeric.";
   
	      
   /*--------------------------- End Fields ---------------------------*/
   
   /*---------------- Begin Convenience Prefs Methods -----------------*/
   
   /** <p>This method returns the local factory from the activity to which this 
       UI is assigned.</p>
   */
   
   public UIFactory getFactory () {
      return uiActivity. localFactory;
   }
   
   /** <p>Convenience method to get Prefs.ACCESS for this recipient and test  
       for equality to Prefs.READ_ONLY.</p>
   */
   
   public boolean isReadOnly () {
      return Prefs. DISPLAY. isReadOnly (this);
   }
   
   /** <p>Convenience method to get the Prefs.TITLE preference for this 
       recipient.</p>
   */
   
   public String getTitle () {
      String title = Prefs. TITLE. getTitle (this);
      CompositeUI enclUI = enclosingUI;
      while (enclUI != null && enclUI. qualifyChildrenTitles) {
         title = Prefs. TITLE. getTitle (enclUI) + " " + title;
         enclUI = enclUI. enclosingUI;
      }
      return title;
   }

   /*----------------- End Convenience Prefs Methods ------------------*/
   
   /*------------------- Begin Presentation Methods -------------------*/
   
   /** <p>This recipient is the domain model object whose presentation is being  
       managed by this CompositeUI instance.  If composite, its field values (along 
       with invalid input values) are displayed during iterative validation, or during 
       presentation initialization.  During instance initialization of this class, a 
       client can provide the recipient object or the recipient class.  If the recipient 
       class is provided, then the recipient object is created internally before rendering 
       for the first time using a parameterless constructor.</p> 
   */
   
   public Object getRecipient () {
      return recipient;
   }
   
   /** <p>Returns the concatenated string of all messages posted for this 
       recipient.  Additional html formatting may be added surrounding and in 
       between these messages.</p>
   */
   
   public String renderValidation () {
      String validation = concatenateValidation ();
      if (validation != null && ! validation. equals (""))
         return "<div class = \"message\">" + validation + "</div>";
      else return "";
   }
   
   /** <p>This method renders only the title (and any validation messages) for this 
       recipient.  The value of the recipient is not rendered.</p>
   */
    
   public String renderTitleAndValidation () {
      return renderTitle () + ":" + renderValidation (); 
   }
   
   /** <p>This method renders only the title (without any validation messages) for this 
       recipient.  If there is a pop-up help page preference for this UI, then this title 
       is hyperlinked to that pop-up window.  If there is a tool tip preference for this UI, 
       then this title is rendered so that the tool tip appears on mouse overs.  A title may 
       have both a pop-up help page and a tool tip. The value of the recipient is not rendered.</p>
   */
    
   public String renderTitle () {

      String popUp = (String) Prefs. POP_UP. get (this);
      String toolTip = Prefs. TOOLTIP. getToolTip (this);
      String title = getTitle ();
      return renderTitle (title, popUp, toolTip);
   }
   
   /** <p>This method renders a title as described in {@link #renderTitle()}, except that the 
       title is for the specified orphan field, that is, a field that is not necessarily 
       directly enclosed in this UI's recipient.  An enclosing UI is not needed in this 
       situation since none of the preferences needed to render a title are inheritable.</p>
   */
   
   public String renderTitle (Field field) {
      String popUp = (String) Prefs. POP_UP. get (field, this);
      String toolTip = Prefs. TOOLTIP. getToolTip (field, this);
      String title = Prefs. TITLE. getTitle (field, this);
      return renderTitle (title, popUp, toolTip);
   }
   
   /** <p>This method renders this recipient using Html.  This rendering typically includes 
       the title, value of the recipient and any validation messages as well.</p> 
   */
   
   public abstract String render ();
   
   /** <p>This method renders only the value of the recipient.</p> */
    
   public abstract String renderVal ();
   
   /** <p>This method implements {@link #renderTitle()}.</p> */
   
   protected String renderTitle (String title, String popUp, String toolTip) {
   
   	  /* See http://www.htmlcodetutorial.com/linking/linking_famsupp_72.html for a quick 
   	     tutorial on pop-up windows in Html with JavaScript.
   	     
   	     See http://www.siteexperts.com/tips/elements/ts10/page1.asp for a quick tutorial 
   	     on tool tips with Html and JavaScript.
   	  */

      if (popUp != null) {
         String html = "<a href = \"" + popUp + "\" onClick = \"return popUp(this, '" + title + "')\" ";
      	 if (toolTip != null) {
            html += "onmouseover = \"this._tip='" + formatToolTip (toolTip) + "'\" ";
         }
		 html += ">" + title + "</a>";
         return html;
      }
      else if (toolTip != null) {
      	String html = "<div " + "onmouseover = \"this._tip='" + formatToolTip (toolTip) + "'\">" + title + "</div>";
      	return html;
      }
      else return title; 
   }
   
   /** <p>This method formats a tool tip message in Html.  Override this method as necessary.</p> */
   
   protected String formatToolTip (String toolTip) {
   	
      if (Prefs. IS_REQUIRED. isTrue (this))
         toolTip = MISSING_REQUIRED + " " + toolTip;
      
      return "<p class = tipBoxContent>" + toolTip + "</p>";
   }
    
   /** <p>This is the public standard official method to validate the user 
       input for a UI's recipient.  This method validates the newly-received 
       user input which corresponds to this recipient.  If the user input is 
       invalid, then it is kept in the UI for redisplay and re-editing while 
       one or more validation messages are posted with this UI.  If the user 
       input is valid, then it is converted to the type of the recipient and 
       stored in the recipient. It returns true if the user input for this 
       recipient validated correctly.  If an active UI was previously validated 
       and has not received additional input, then this method returns true but 
       does not validate again.</p>
   */
   
   public abstract boolean validate (PageContext pageContext);

   /** <p>This method returns false only when the last input for this recipient was not 
       valid.</p>
   */
   
   public boolean isValid () { return valid; }
   
   /** <p>Returns true if input for this UI (whether elementary or composite) was submitted 
       the last time a form for this activity was submitted. It is useful to track down input 
       submission at the elementary level because the fields requested during composite object 
       input may change from validation iteration to iteration, for example when using the 
       {@link com.eknowlogie.aliorsum.prefs.Prefs#WHEN_INVALID WHEN_INVALID} preference.</p>
   */
   
   public abstract boolean wasInputSubmitted ();
   
   /** <p>Returns the concatenated string of all messages posted for this 
       recipient.  This method may add html formatting in between these 
       messages.</p>
   */
   
   protected String concatenateValidation () {
      if (validationMessages == null) {
         return "";
      }
      else {
         String text = "";
         Iterator it = validationMessages. iterator ();
         while (it. hasNext ()) {
            text += (String) it. next ();
            if (it. hasNext ()) 
               text += " ";
         }
         return text;
      }
   }
   
   /** <p>Adds a validation message "msg" for this recipient.  Any number of 
       messages may be posted for this recipient.  These messages do not 
       have to be all remedial validation messages, they can also be preventive, 
       initial explanatory messages describing input format rules, etc.  These 
       messages are cleared when a new input submission for this recipient is 
       received from the user.</p>
   */
   
   public void addValidation (String msg) {
      if (validationMessages == null) {
         validationMessages = new HashSet ();
      }
      validationMessages. add (msg);
   }

   /** <p>This is the unique qualified name used to identify this recipient in Html 
       forms during rendering.  It is also used to post and retrieve this UI in and  
       from the list of active UIs in the activity.</p>
   */

   public String getQualifiedName () { return qualifiedName; }
   
   /** <p>This is the page context being replaced by a new page context for JSP output 
       collection.</p>
   */
   
   private PageContext originalPageContext = null;
   
   /** <p>This is the new jsp body pushed unto the page context stack to collect subsequent 
       JSP output into a string.</p>
   */
   
   private BodyContent collectedOutput = null;
   
   /** <p>This method is used inside a JSP page to begin collecting the JSP output into 
       a local string inside this UI.  The JSP's local <code>out</code> variable should 
       receive this return value.</p>
   */
   
   public JspWriter beginOutputCollection (PageContext pageContext) {
      originalPageContext = pageContext;
      collectedOutput = originalPageContext. pushBody ();
      return collectedOutput;
   }
   
   /** <p>This method is used inside a JSP page to end the collection of JSP output into 
       a local string inside this UI.  The JSP's local <code>out</code> variable should 
       receive this return value.</p>
   */
   
   public JspWriter endOutputCollection () {
      JspWriter w = originalPageContext. popBody ();
      originalPageContext = null;
      return w;
   }

   /** <p>This method returns a string containing the output from the JSP page with file name 
       <code>jspPage</code>.  This method performs a dynamic server-side include of that given 
       JSP page and it should be used whenever a JSP output is to be used by methods performing 
       rendering operations.  Otherwise, allowing the JSP output to go directly to the page 
       output can cause the output sequence to be altered as the include page output will go to 
       the page output before the rest of the string output being prepared by the rendering 
       methods performing the server-side include.  An included JSP page must be bracketed by 
       {@link #beginOutputCollection(PageContext)} and {@link #endOutputCollection()} so that 
       its output can be redirected into this UI.</p>
   */
   
   public String include (String jspPage) {
      PageContext pageContext;
      if (this instanceof CompositeUI)
         pageContext = ((CompositeUI) this). getCurrPageContext  ();
      else pageContext = enclosingUI. getCurrPageContext ();
      
      try { pageContext. include (jspPage); } // TODO TEMP need absolute paths
      catch (Exception e) { throw new Error (e. getMessage ()); }
      
      if (collectedOutput == null) 
         throw new Error ("Its likely that JSP page " + jspPage + " does not have begin-end output collection brackets.");
      
      String result = collectedOutput. getString ();
      collectedOutput = null;
      return result;
   }
      

   /*-------------------- End presentation Methods ---------------------*/

   /* End of UI */
}