/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum;

import java.util.Stack;
import java.util.Hashtable;
import java.util.Vector;
import java.lang.reflect.Field;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.PageContext;
import com.eknowlogie.util.DeferredHashTable;
import com.eknowlogie.aliorsum.prefs.*;


/** <p>A UIActivity is a disposable UI container which holds all the composite and 
    elementary UIs in use throughout a given presentation activity and can provide  
    validation control throughout the activity. The life-span of a presentation activity  
    is independent of a page or request and can spread arbitrarily over multiple pages 
    or requests. Contrary to other frameworks like Struts, this allows for the continuity
    of partially-filled or invalid inputs to be preserved across multiple pages.  This 
    class also provides support for limited page control flow during the validation 
    process of the activity.  For example, it offers a mechanism for a wizard-style 
    input page sequence.</p>
    
    <p>An activity also has a local UI factory, which can be used to create and access 
    rendering-specific, activity-specific cascading preferences.  This local factory
    is created using the general session factory as its subordinate, this way, preference 
    selection automatically cascades into the {@link UIFactory#subordinate} factory.  
    For example, if we wanted to hide field "shippingInfo" in a rendering of an instance 
    of its enclosing class Order, we would register a one-time activity-specific 
    preference in the usual way  as follows:</p>
    
    <pre>
    if (UIActivity. isFirstTime (pageContext)) {
       UIActivity. fieldPrefs (). register ("shippingInfo", Order. class, 
                                            Prefs. ACCESS, Access. HIDDEN);
    }
    </pre>
    
    <p>The use of an activity as the current session activity is delimited by bracketed 
    static calls to begin and end the activity and may be discarded afterwards. Thus 
    there is at most one session activity at one time.  If otherwise ignored, these 
    activities become transient and unreferenced after use.  Alternatively, these 
    activities can be created and pre-configured ahead of time so that they can be reused 
    time and time again for specific situations.  The use of an activity is managed through 
    static methods, for the following reasons: </p>
    
    <ul>
    
       <li><p>This way we can enforce that the current session activity is the one 
       actually manipulated throughout the activity.</p>
       </li>
       
       <li><p>The static methods can better manage the conditional situations leading 
       to activity instantiation and begin and end of activities, avoiding repetitive 
       conditional code from permeating the client presentation.</p>
       </li>
       
       <li><p>Pre-configuration of reusable activities still can be done with instance 
       counterpart of the applicable static methods.</p>
       </li>
       
    </ul>

    @author Romel Rivera, eKnowlogie. Inc.
*/

public class UIActivity {

   /*-------------------------- Begin Fields --------------------------*/

   /** <p>This is the stack of the active composite UI's currently being rendered 
       by the servlet.  JSP renderers will maintain the stack by using CompositeUI's 
       beginUIRendering and endUIRendering brackets.  This stack behavior is used 
       to produce unique qualified names for active UIs.</p>
   */
	
   protected Stack uiRenderingStack ;
   
   /** <p>These are the elementary and composite UIs which are active or in use by this 
       activity; they are retrieved by the unique qualified name they were assigned to 
       before rendering the first time.  The entries are as follows: {String qualifiedName, 
       UI activeUI).  These active UIs are also kept in a tree denoting their nested 
       structure (elementary UIs are the leafs of the tree).  This tree is formed through 
       begin-end rendering brackets, and it is used to recursively validate the active UIs.</p>
   */
   
   protected Hashtable activeUIs;

   /** <p>Only the current activity and the subordinate or session factory are the 
       Aliorsum components stored in the Http session.  This way, the life-time of an 
       activity can be maintained across multiple requests.</p>
   */
   
   protected HttpSession session;
   
   /** <p>Using this variable is the mechanism to bypass the default naming for active 
       composite UIs.</p>

       @see #setNextName(String)
   */
   
   protected String nextName;
   
   /** <p>This is the root centinel for the active UI tree structure maintained 
       during the activity.</p>
   */
   
   protected CompositeUI root;   
   
   /** <p>This is the local factory for this activity.  The local factory is 
       used to create and access rendering-specific cascading preferences.  This 
       local factory is created (or initialized later) to use the general session 
       factory as its subordinate, this way, preference selection automatically 
       cascades into the subordinate factory.</p>
   */
   
   public UIFactory localFactory;
   
   /** <p>During the course of an activity, the user may move away from partially-filled   
       components to other sections or pages in the activity, just to come back and resume 
       input later.  If this flag is true, any validation messages reflecting the current 
       input state of those components will be displayed on arrival back to that section or 
       page after the hiatus.  If false, no messages will be displayed until the user elects  
       to submit the components input as completed.  It is false by default, so that the  
       presentation can remain unintrusive to the user moving about and partially filling 
       forms throughout the activity, allowing the user to first signal completion of a form  
       or activity section before providing a validation response.
   */
   
   public boolean displayValidationAfterHiatus = false;
	      
   
   /** <p>The rendering name of the tree root centinel of active UIs.</p> */
   
   private static final String rootName = "";
   
   /** <p>The Http session name for the current activity.</p> */
   
   private static final String activityName = "uiActivity";
   
   /*--------------------------- End Fields ---------------------------*/
   
   /*----------------- Begin Validation Control Fields ----------------*/
   
   /** <p>This is the page where control will be forwarded upon successful 
       validation after a "submit" button submission.</p>
   */
   
   protected String successPage = null;
   
   /** <p>This is the page where control will be forwarded upon cancelation 
       of the activity with the "cancel" button.</p>
   */
   
   protected String failurePage = null;
   
   /** <p>This is the wizard-style linear page sequence making up an activity.  
       Upon submission of buttons "next" and "previous", these pages will be 
       visited (control forwarded to) in sequence.</p>
   */
   
   protected Vector pageSequence = null;
   
   /** <p>This index represents the current page we are executing in the 
       page sequence.</p>
   */
   
   protected int currentPageIndex = 0;
	      
   /*------------------ End Validation Control Fields -----------------*/

   /*----------------------- Begin Constructors -----------------------*/
   
   /** <p>Creates a new UI activity and stores it in the session as the current 
       activity in use.  A new activity with this constructor is created automatically 
       when an activity is begun or when there is no activity and one is first needed.</p>
   */
   
   protected UIActivity (PageContext pageContext) {
      this ();
      init (pageContext);
   }
   
   /** <p>Creates a new activity without setting it up as the current session 
       activity in use.  This constructor is useful for creating and configuring 
       activities in advance so that they can be reused in specific situations.</p>
   */
   
   public UIActivity () {
      localFactory = new UIFactory (true);
   }
   
   /*------------------------ End Constructors ------------------------*/

   /*---------------------- Begin Static Methods ----------------------*/
   
   /** <p>Retrieves the current UI activity from the session.  It does not create a new 
       one if one does not exists, because we want the life-time of the session activity 
       to be managed explicitly.  If raises an exception if no activity has been initiated. 
       </p>
   */
   
   public static UIActivity getUIActivity (PageContext pageContext) {
      UIActivity uiActivity = (UIActivity) pageContext. getSession (). getAttribute (activityName);
      if (uiActivity == null) {
      	throw new Error ("No activity has been initiated for this rendering.");
      }
      return uiActivity;
   }
   
   /** <p>This method is used as a reference to add field preferences in the local 
       factory of this composite UI object.  The notation "getFieldPrefs ()" is 
       not used, to clarify that this is not a regular "getter" used to inspect 
       value but instead it is used when adding preferences to the table and 
       may cause the table to be created making it more expensive for subsequent 
       searches.</p>
   */
   
   public static FieldPrefsRegistry fieldPrefs (PageContext pageContext) {
      UIActivity uia = getUIActivity (pageContext);
      return uia. fieldPrefs ();
   }
   
   /** <p>This method is used as a reference to add class preferences in the local 
       factory of this composite UI object.  The notation "getClassPrefs ()" is 
       not used, to clarify that this is not a regular "getter" used to inspect 
       value but instead it is used when adding preferences to the table and 
       may cause the table to be created making it more expensive for subsequent 
       searches.</p>
   */
   
   public static ClassPrefsRegistry classPrefs (PageContext pageContext) {
      UIActivity uia = getUIActivity (pageContext);
      return uia. classPrefs ();
   }
   
   /** <p>This method is used as a reference to add default preferences in the local 
       factory of this composite UI object.  The notation "getDefaultPrefs ()" is 
       not used, to clarify that this is not a regular "getter" used to inspect 
       value but instead it is used when adding preferences to the table and 
       may cause the table to be created making it more expensive for subsequent 
       searches.</p>
   */
   
   public static DeferredHashTable defaultPrefs (PageContext pageContext) {
      UIActivity uia = getUIActivity (pageContext);
      return uia. defaultPrefs ();
   }
   
   
   /** <p>Convenience method to turn name into a readable prefix (by appending 
       "_") in order to use it as a qualifier to create unique qualified names.</p>
   */
   
   public static String asPrefix (String name) {
      if (name == null || name. equals (""))
         return "";
      else return name + "_";
   }

   /** <p>This is the standard public method used to validate all the newly-received 
       input for this activity.  Invalid input fields and validation messages are kept 
       in the corresponding UIs while validated input fields are stored in the recipients. 
       It returns true if all input data was valid.  In a multi-page activity, this method 
       can be used to validate in stages, because it will automatically only validate the  
       UIs for which new input has been received (input form the current page). If an active 
       UI was previously validated and has not received additional input, then it is not 
       reprocessed again for validation.  It invokes {@link CompositeUI#validate(PageContext)} 
       to perform the validation for the composite UI tree.</p>
   */
   
   public static boolean validate (PageContext pageContext) {
      UIActivity uia = getUIActivity (pageContext);
      return uia. validate_Instance (pageContext);
   }
   
   /** <p>The use of an activity as the current activity is delimited by bracketed 
       static calls to begin and end the activity and may be discarded afterwards. 
       Thus there is at most one session activity at one time.  This method creates 
       a new activity and makes it the new current session activity in use.  Given 
       that pages within an activity may have other activity-unrelated hyperlinks 
       which jump outside the activity, it may not always be possible to surround an 
       activity with begin and end brackets.  A <code>begin</code> activity bracket 
       is mandatory to initiate activity but the <code>end</code> activity bracket 
       is optional.</p>
   */
   
   public static UIActivity begin (PageContext pageContext) {
      return new UIActivity (pageContext);
   }
   
   /** <p>Convenience method to initiate a new activity at the time the first page 
       in the activity is hyper-linked.  This method returns the url so that it 
       can be used as the attribute value for "href".</p>
   */
   
   public static String begin (PageContext pageContext, String url) {
      begin (pageContext);
      return url;
   }
   
   /** <p>This method reuses an old activity by making it the new current session 
       activity in use.</p>
       
       @see #begin(PageContext)
   */
   
   public static UIActivity begin (PageContext pageContext, UIActivity oldActivity) {
      oldActivity. init (pageContext);
      return oldActivity;
   }
   
   /** <p>This method ends the use of the current session activity by removing  
       it from the session.</p>
       
       @see #begin(PageContext)
   */
   
   public static UIActivity end (PageContext pageContext) {
      UIActivity uia = (UIActivity) pageContext. getSession (). getAttribute (activityName);
      if (uia != null) {
      	pageContext. getSession (). removeAttribute (activityName);
      }
      return uia;
   }
   
   /** <p>Convenience method to obtain the current elementary UI being processed. </p>
    
       @see CompositeUI#currElementaryUI
   */
   
   public static ElementaryUI getCurrElementaryUI (PageContext pageContext) {
      UIActivity  uia = getUIActivity (pageContext);
      return uia. getCurrCompositeUI (). currElementaryUI;
   }
   
   /** <p>This method displays a toggle Html button to allow the user to switch between 
       redisplaying only invalid fields (those with validation error messages) and 
       redisplaying all fields (valid or invalid) during an input validation iteration</p>.
   */
   
   public static String displayInvalidToggle (PageContext pageContext) {
      UIActivity  uia = getUIActivity (pageContext);
      return uia. displayInvalidToggle_Instance (pageContext);
   }
   
   /*----------------------- End Static Methods -----------------------*/
   
   /*------------- Begin Static Validation Control Methods ------------*/
   
   /** <p>This method indicates that the validation control flow of this single-page 
       activity will be managed by the activity itself.  This method assumes the use 
       of standard submission buttons as described in {@link #setControl(PageContext, 
       String, String, String[]) setControl}, except for "next" and "previous".</p>
   */
   
   public static UIActivity setControl (PageContext pageContext, String successPage, String failurePage) {
      UIActivity uia = getUIActivity (pageContext);
      return uia. setControl (successPage, failurePage);
   }
   
   /** <p>This method indicates that the validation control flow of this multi-page activity 
       will be managed by the activity itself, as a <b>wizard-style</b> linear sequence.  This 
       form of control assumes a predefined role for the following submission button names:</p>
       
       <ul>
          <li> <p><b>submit</b>. If the activity validates all its input successfully,  
               then control jumps to the success page, otherwise control stays on the 
               same page.  This button can be used only on sequence-ending pages (i.e. 
               similar to the "Finish" button in wizards).</p>
          </li>
          
          <li><p><b>cancel</b>.  With this button, input is not validated and control is 
               forwarded to the failure page.</p>
          </li>
          
          <li><p><b>next</b>.  If the page validates successfully, control is forwarded to 
              the next page in the sequence, otherwise control stays on the same page.</p>
          </li>
          
          <li><p><b>previous</b>.  With this button the current page input is not validated 
              (but kept) and control is forwarded to the previous page.</p>
          </li>
          
          <li><p><b>redisplay</b>.  This button is used to toggle between the display of 
              all fields in the page and the display of only the fields with invalid input. 
              This is a convenience button for users to allow them to more easily locate 
              the fields with invalid input.  This button can be placed on a page by 
              invoking {@link #displayInvalidToggle(PageContext)} from the JSP page.</p>
          </li>
          
          <li><p><b>stay</b>.  This is a button (i.e. a query parameter) used internally 
              by the activity in order to control the sequence.</p>
          </li>
       </ul>
   */
   
   public static UIActivity setControl (PageContext pageContext, 
   		                                String successPage, 
										String failurePage,
										String [] pageSequence) 
   {
      UIActivity uia = getUIActivity (pageContext);
      return uia. setControl (successPage, failurePage, pageSequence);
   }
   
   /** <p>This method validates input and takes possible control flow action by 
       potentially forwarding the request to another page depending on the control 
       guidelines currently enacted.  See {@link #setControl(PageContext, String, String)}
       and {@link #setControl(PageContext, String, String, String[])} for setting 
       up the control guidelines.  This method returns true only if control has been 
       forwarded elsewhere.  If this method returns true, then the JSP page must likely 
       follow with a <code>return</code>.</p>
   */
   
   public static boolean validateAndForward (PageContext pageContext) {

   	  UIActivity uia = getUIActivity (pageContext);
   	  return uia. validateAndForward_Instance (pageContext);
   }
   
   /** <p>This method checks to see if we are initiating an input activity and therefore 
       have not received any input from the user yet. </p>
   */
   
   public static boolean isFirstTime (PageContext pageContext) {
   	  HttpServletRequest request = (HttpServletRequest) pageContext. getRequest ();
      boolean firstTime = request. getParameter ("formSubmitted") == null;
      return firstTime;
   }
   
   /*-------------- End Static Validation Control Methods -------------*/

   /*------------------------- Begin Methods --------------------------*/
   
   /** <p>This method initializes a UIActivity so that it can be used as the 
       current session activity.  It binds its subordinate factory to the session 
       factory.</p>
   */
   
   protected void init (PageContext pageContext) {
      this. session = pageContext. getSession ();
      session. setAttribute (activityName, this);
      localFactory. subordinate = UIFactory. getUIFactory (pageContext);
      if (localFactory. subordinate == null) {
         throw new Error ("Session factory has not been initialized.");
      }
      root = new CompositeUI ();
      uiRenderingStack = new Stack ();
      uiRenderingStack. push (root);
      currentPageIndex = 0;
      activeUIs = new Hashtable ();
      nextName = null;
      root. init (new Object (), Object. class, pageContext, rootName);
   }
   
   /** <p>This mehod retrieves an active composite UI from the current UI activity 
       by using its simple name and the assumption that this UI lies nested inside 
       the composite UI currently being rendered (which in turn supplies its name as  
       the prefix to form the fully-qualified name of the UI being retrieved).</p>
   */
   
   protected CompositeUI getActiveCompositeUI (String simpleName) {
      String qualifiedName = qualifyName (simpleName);
      return (CompositeUI) activeUIs. get (qualifiedName);
   }
   
   
   /** <p>This mehod retrieves an active UI from the current UI activity by using 
       its simple name and the assumption that this UI lies nested inside the composite 
       UI currently being rendered (which in turn supplies its name as the prefix to 
       form the fully-qualified name of the UI being retrieved).</p>
   */
   
   protected UI getActiveUI (String simpleName) {
      String qualifiedName = qualifyName (simpleName);
      return (UI) activeUIs. get (qualifiedName);
   }
   
   /** <p>This mehod retrieves an active elementary UI from the current UI activity 
       by using its simple name and the assumption that this UI lies nested inside 
       the composite UI currently being rendered (which in turn supplies its name as  
       the prefix to form the fully-qualified name of the UI being retrieved). If an 
       active elementary UI has not yet been assigned to this field recipient, one 
       is created.  The notation "getActiveElementaryUI" is not used to denote that 
       this method is also a setter.</p>
   */
   
   protected ElementaryUI activeElementaryUI (Field field, CompositeUI parentUI) {
   	  String qualifiedName = qualifyName (field. getName ());
      ElementaryUI eui = (ElementaryUI) activeUIs. get (qualifiedName);
      if (eui == null) {
         eui = localFactory. createElementaryUI (field, parentUI);
      }
      return eui;
   }
   
   /** <p>This method is equivalent to {@link #activeElementaryUI(Field, CompositeUI)} except 
       that this elementary recipient is not a field within the current composite UI.  This 
       is useful for rendering elementary objects which are unrelated to the object model 
       structure being rendered, such as temporary or derived presentation values.</p>
   */
   
   protected ElementaryUI activeElementaryUI (Object recipient, String simpleName, CompositeUI parentUI) {
      String qualifiedName = qualifyName (simpleName);
      ElementaryUI eui = (ElementaryUI) activeUIs. get (qualifiedName);
      if (eui == null) {
         eui = localFactory. createElementaryUI (recipient, simpleName, parentUI);
      }
      return eui;
   }
   
   /** <p>This method is the instance version of static {@link #fieldPrefs(PageContext)} 
       so that it can be used during pre-configuration of reusable activity objects.</p>
   */
   
   public FieldPrefsRegistry fieldPrefs () {
      return localFactory. fieldPrefs ();
   }
   
   /** <p>This method is the instance version of static {@link #classPrefs(PageContext)} 
       so that it can be used during pre-configuration of reusable activity objects.</p>
   */
   
   public ClassPrefsRegistry classPrefs () {
      return localFactory. classPrefs ();
   }
   
   /** <p>This method is the instance version of static {@link #defaultPrefs(PageContext)} 
       so that it can be used during pre-configuration of reusable activity objects.</p>
   */
   
   public DeferredHashTable defaultPrefs () {
      return localFactory. defaultPrefs ();
   }
   
   /** <p>This method is the instance version of {@link #setControl(PageContext, String, String)} 
       so that it can be used during pre-configuration of reusable activity objects.</p>
   */
   
   public UIActivity setControl (String successPage, String failurePage) {
      this. successPage = successPage;
      this. failurePage = failurePage;
      return this;
   }
   
   /** <p>This method is the instance version of {@link #setControl(PageContext, String, String, String[])} 
       so that it can be used during pre-configuration of reusable activity objects.</p>
   */
   
   public UIActivity setControl (String successPage, 
							     String failurePage,
								 String [] pageSequence) 
   {
      this. successPage = successPage;
      this. failurePage = failurePage;
      this. pageSequence = new Vector ();
      for (int ix = 0; ix < pageSequence. length; ix ++ ) {
         this. pageSequence. add (pageSequence [ix]);
      }
      currentPageIndex = 0;
      return this;
   }

   /** <p>Returns the qualified name assigned for rendering, to the 
       composite UI currently being rendered.  Naming active composite UIs 
       is necessary in order to be able to retrieve them by their 
       unique name and in order to use their names to uniquely qualify  
       any form fields during the input and validation process.</p>
   */
   
   public String getCurrQualifiedName () {
      if (uiRenderingStack. isEmpty ()) 
         return "";
      else return getCurrCompositeUI (). qualifiedName;
   }

   /** <p>Prefixes a simple name with the unique qualified name corresponding 
       to the current UI component being rendered.  If "nextName" is not null, 
       it uses nextName instead of simpleName as the mechanism for bypassing 
       default naming conventions.  It is used internally by Gaviidae to post 
       and retrieve active UIs from the activity.</p>
   */
   
   public String qualifyName (String simpleName) {
   
      if (nextName != null) {
      	 simpleName = nextName;
      }
      
      String qualifiedName = asPrefix (getCurrQualifiedName ()) + simpleName;
      return qualifiedName;
   }
   
   /** <p>Presets the value which will override the next simple name being submitted for 
       qualification.  This is used to bypass default naming conventions.  It is used automatically  
       by the {@link CompositeUI#presetUIRendering(Object, PageContext, String) presetUIRendering}
       methods.</p>
   */
   
   public void setNextName (String name) {
      nextName = name;
   }
   
   /** <p>Clears the value of  nextName so that it may no longer be used to override a subsequent 
       simple name.  It is used internally by the {@link CompositeUI#beginUIRendering(Object, 
       PageContext, String) beginUIRendering} methods.</p>
   */
   
   public void clearNextName () {
      nextName = null;
   }
   
   /** <p>Returns the current composite UI being rendered.</p> */
   
   public CompositeUI getCurrCompositeUI () {
      return (CompositeUI) uiRenderingStack. peek ();
   }
   
   /** <p>This is the instance version of the static method 
       {@link #validate(PageContext)}.</p>
   */
   
   private boolean validate_Instance (PageContext pageContext) {
      return root. validate (pageContext);
   }
   
   /** <p>This is the instance version of {@link #validateAndForward(PageContext)}, so that 
       it can be overriden.</p> 
   */
   
   protected boolean validateAndForward_Instance (PageContext pageContext) {
   	
   	  /* This method uses sendRedirect() instead of forward() when transfering control 
   	     after the activity has ended, in order not to carry over all the query parameters 
   	     from this activity on to the next activity.
   	  */

   	  HttpServletRequest request = (HttpServletRequest) pageContext. getRequest ();
   	  HttpServletResponse response = (HttpServletResponse) pageContext. getResponse ();
   	  
   	  try {
         if (request. getParameter ("stay") != null) { }
         else if (request. getParameter ("redisplay") != null) {
         	validate (pageContext);
            boolean whenInvalid = defaultPrefs (). getBoolean (Prefs. WHEN_INVALID);
            defaultPrefs (). put (Prefs. WHEN_INVALID, new Boolean ( ! whenInvalid));
         }
         else if (request. getParameter ("cancel") != null) {
         	 response. sendRedirect (failurePage);
         	 return true;
         }
         else {
         	boolean validated = validate (pageContext);
         	if (request. getParameter ("submit") != null) {
         	   if (validated) {
      	         response. sendRedirect (successPage);
      	         return true;
      	       }
      	    }
      	    else if (request. getParameter ("previous") != null) {
      	       currentPageIndex --;
               response. sendRedirect (pageSequence. get (currentPageIndex) + "?stay=true");
               return true;
         	}
     	    else if (request. getParameter ("next") != null) {
     	       if (validated) {
      	          currentPageIndex ++;
                  response. sendRedirect (pageSequence. get (currentPageIndex) + "?stay=true");
                  return true;
     	       }
      	    }
          }
         return false;
   	  }
      catch (Exception e) { throw new Error (e. getMessage ()); }
   }
   
   /** <p>This is the instance version of {@link #displayInvalidToggle(PageContext)} so that it can 
       be overriden.</p>
   */
   
   protected String displayInvalidToggle_Instance (PageContext pageContext) {
   	
  	  HttpServletRequest request = (HttpServletRequest) pageContext. getRequest ();
      if (isFirstTime (pageContext) || request. getParameter ("stay") != null) { 
         return "";
      }
      
      boolean whenInvalid = defaultPrefs (). getBoolean (Prefs. WHEN_INVALID);
      String toggleButton = "<input type = \"submit\" name = \"redisplay\" value = \"";
	  if (whenInvalid)
	     toggleButton += "Display All Fields";
	  else toggleButton += "Display Only Invalid Fields";
      return toggleButton + "\" >";
   }

   /*-------------------------- End Methods ---------------------------*/

   /* End of UIActivity */
}