/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum;

import java.util.Hashtable;
import java.lang.reflect.Field;
import javax.servlet.jsp.PageContext;
import com.eknowlogie.util.DeferredHashTable;
import com.eknowlogie.aliorsum.prefs.Pref;
import com.eknowlogie.aliorsum.prefs.Prefs;
import com.eknowlogie.aliorsum.primitive.PrimitivePrefs;

/** <p>A UIFactory is the class responsible for 1) storing and selecting cascading  
    preferences, and 2) creating the elementary and composite UIs chosen through these 
    preferences.</p>
    
    <p>Preference registration in a factory is supported through three levels of 
    generality (from specific to general):</p>
    
    <ul>
       <li><p>Field-Specific Preferences. Preferences at this level are registered to 
       define how specific fields in a domain model class must be presented.  Field 
       preferences supersede the class preferences assigned to the class type of that 
       given field.  See {@link FieldPrefsRegistry} for details.</p>
       </li>
       
       <li><p>Class-Specific Preferences.  Preferences at this level define how 
       specific classes in the domain model must be presented.  They supersede 
       default preferences.  See {@link ClassPrefsRegistry} for details.</p>
       </li>
       
       <li><p>Default Preferences. These are the preference settings to be used if 
       no other settings are specified for a given object.</p>
       </li>
    </ul>
    
    <p>Factories can also have different life-times:</p>
    
    <ul>
       <li><p>A <b>session factory</b> is available throughout the entire user 
       session.</p>
       </li>
       
       <li><p>An <b>activity factory</b> is available locally during a given activity.</p>
       </li>
    </ul>
    
    <p>A session factory is typically <b>subordinated</b> to an activity factory. Both 
    local and session factories support all preference levels of generality.  This 
    structure above provides support for a <b>cascading preference selection</b> 
    mechanism (see {@link #getPref(Field, Pref, UI) getPref}) where a preference value  
    is first looked up in the local or activity preferences (throughout all its levels 
    of generality, from specific to general), and if not found, looked up in the session 
    preferences (also throughout all its levels of generality).</p>
    
    <p><b>Creating elementary and composite UI instances</b>.  While preference 
    selection is done by a factory, core preference interpretation is done by the  
    individual preference classes except for two preferences which are managed by 
    the factory.  They are Prefs.ELEMENTARY_UI_CLASS and Prefs.COMPOSITE_UI_CLASS, 
    whose Class values are used to create instance of such classes to host the 
    presentation of recipients.</p>

    @author Romel Rivera, eKnowlogie. Inc.
*/

public class UIFactory {

   /*-------------------------- Begin Fields --------------------------*/
	
   /** <p>These are the field preferences for this factory.</p>
    
       @see FieldPrefsRegistry
   */
	
   private FieldPrefsRegistry fieldPrefs = new FieldPrefsRegistry ();
   
   /** <p>These are the class preferences for this factory.</p>
    
       @see ClassPrefsRegistry
   */
	
   private ClassPrefsRegistry classPrefs = new ClassPrefsRegistry ();
   
   /** <p>These are the default preferences for this factory.  This table 
       is a map (Pref prefId -> Object prefValue).  Under Java 5.0, this 
       typing should be made explicit.</p>
   */
   
   private DeferredHashTable defaultPrefs = new DeferredHashTable ();
   
   /** <p>This is the subordinate factory for this factory.  Factory cascading 
       selection of preferences, if not successful, is passed to its subordinate 
       factory.</p>
   */
   
   public UIFactory subordinate = null;
   
   /** <p>If true, this flag indicates that during class-specific preference selection in  
       this factory, the search should continue on to the superclasses before cascading 
       to the default preferences.  Typically this flag will be true only for the session 
       factory. </p>
       
       @see #UIFactory(boolean)
   */
   
   public boolean cascadeToSuperClasses = false;
   
   /** <p>If true, this flag indicates that during class-specific preference selection in  
       this factory, the search should continue up the object structure, to the classes of 
       the recipient's enclosing objects, before cascading to the default preferences.  As 
       implemented, it is always true.</p>
   */
   
   public boolean cascadeToEnclosingObjects = true;
   
   /** <p>This field provides access to class information such as the list 
       of elementary and composite fields for a given class.</p>
   */
   
   private Classes classes;
   
   /** <p>This is the Http session name to be used for the session factory.</p> */
   
   public static final String SESSION_NAME = "UIFactory";
	      
   /*--------------------------- End Fields ---------------------------*/
   
   /*----------------------- Begin Constructors -----------------------*/
   
   /** <p>Creates a UI factory with a subordinate.  If the subordinate is 
       null, this is assumed to be a session factory and its core preferences 
       are registered here.</p>
   */
   
   public UIFactory (UIFactory subordinate) {
      this (subordinate != null);
      if (subordinate != null) 
         this. subordinate = subordinate;
      else throw new Error ("Use another constructor for a UIFactory without a subordinate");
   }
   
   /** <p>Creates a UI factory.  If this factory is a local factory, its subordinate session 
       factory is not yet bound, so that it can be pre-configured for reuse and later bound 
       to the session factory.  If this factory is a session factory, then its core preferences 
       are registered here, and {@link #cascadeToSuperClasses} is set to true.</p> 
   */
   
   public UIFactory (boolean isLocalFactory) {
      if (! isLocalFactory) {
      	 cascadeToSuperClasses = true;
         classes = new Classes (this);
         new Prefs (). initializePrefs (this);
         PrimitivePrefs. register (this);
      }
   }
   
   /*------------------------ End Constructors ------------------------*/

   /*---------------------- Begin Static Methods ----------------------*/
   
   /** <p>There are only two Gaviidae objects maintained in the session, 
       the session factory and the current UI activity.  This method retrieves 
       the factory from the session.  The session factory must be created and 
       posted into an Http session explicitly at the beginning of a session.</p>
   */
   
   public static UIFactory getUIFactory (PageContext pageContext) {
      UIFactory uif = (UIFactory) pageContext. getSession (). getAttribute (SESSION_NAME);
      return uif;
   }
   
   /*----------------------- End Static Methods -----------------------*/
   
   /*--------------- Begin Cascading Preference Methods ---------------*/
   
   /** <p>Returns an object that provides reflective information about 
       classes, such as their elementary and composite fields.</p>
   */
   
   public Classes getClasses () {
   	
   	  if (subordinate != null)
   	  	 return subordinate. getClasses ();
   	  else if (classes == null) {
   	     classes = new Classes (this);
   	  }
      return classes;
   }
   
   /** <p>This method is used as a reference to add field preferences in this  
       factory.  The notation "getFieldPrefs ()" is not used, to clarify that 
       this is not a regular "getter" used to inspect value but instead it is 
       used when adding preferences to the table and may cause the table to be 
       created making it more expensive for subsequent searches.</p>
   */
   
   public FieldPrefsRegistry fieldPrefs () {
      return fieldPrefs;
   }
   
   /** <p>This method is used as a reference to add class preferences in this 
       factory.  The notation "getClassPrefs ()" is not used, to clarify that 
       this is not a regular "getter" used to inspect value but instead it is 
       used when adding preferences to the table and may cause the table to be 
       created making it more expensive for subsequent searches.</p>
   */
   
   public ClassPrefsRegistry classPrefs () {
      return classPrefs;
   }

   /** <p>This method is used as a reference to add default preferences in this 
       factory.  The notation "getDefaultPrefs ()" is not used, to clarify that 
       this is not a regular "getter" used to inspect value but instead it is 
       used when adding preferences to the table and may cause the table to be 
       created making it more expensive for subsequent searches.</p>
   */
   
   public DeferredHashTable defaultPrefs () {
      return defaultPrefs;
   }
   
   /** <p>This method applies <b>cascading preference selection</b> to return the   
       value of preference "preferenceId" for preference receiver "field" enclosed  
       in "enclosingUI".  This preference search cascades to its subordinate factory  
       if necessary.  Cascading preference selection is carried out as follows:</p>
       
       <ul>
          <li><p>Search the generality levels in the local or activity factory.</p>
          </li>
          
          <li><p>If not found, search the generality levels of the subordinate 
              factory, which is typically the session factory.</p>
          </li>
       </ul>
       
       <p>The search through generality levels is done as follows:</p>
             
       <ul>
          <li><p><i>Field-Specific Preferences</i>. Search the field preferences for the 
              given target receiver field.</p>
          </li>
                
          <li><p><i>Class-Specific Preferences</i>. If not found above, search the class 
              preferences using the target receiver class (e.g. the class of the receiver   
              field).</p>
          
              <ul>
                 <li><p><i>Super Class Search</i>. If a preference value is not found, 
                     continue the search using the super classes except Object.</p>
                 </li>
               
                 <li><p><i>Contextual Preference Search</i>.  If a preference value is not 
                     found, and the preference is {@linkplain Pref#isInheritable() inheritable}, 
                     repeat the class search using the classes of the enclosing recipient objects
                     in the UI hierarchy.</p>
                 </li>
              </ul>
          </li>
                
          <li><p><i>Default Preferences</i>. If not found above, search the default preferences.</p>
          </li>
       </ul>
       
       <p>Notice that super class search is controlled with flag {@link #cascadeToSuperClasses}, 
       (true for session factories), while contextual search is controlled with 
       {@link #cascadeToEnclosingObjects} (always true).</p>
       
       @param field        <p>This is the target receiver for which we are selecting a preference.</p>
       
       @param preferenceId <p>The preference for which we are attempting to select a value.</p>
       
       @param enclosingUI  <p>The enclosing UI of the target receiver.  This is useful when inheriting 
                           a preference value from the enclosing object structure, but it can be null.</p> 
                           
       @return             <p>It returns the selected preference value for this target receiver.</p>
   */
   
   public Object getPref (Field field, Pref preferenceId, UI enclosingUI) {
      Object pref = getUnsubordinatedPref (field, preferenceId, enclosingUI);
      if (pref != null)
	     return pref;
      else if (subordinate != null)
         return subordinate. getPref (field, preferenceId, enclosingUI);
      else return null;
   }
   
   /** <p>Same as {@link #getPref(Field, Pref, UI)} except that field searches are 
       omitted.</p>
   */
   
   public Object getPref (Class recipientClass, Pref preferenceId, UI enclosingUI) {
      Object pref = getUnsubordinatedPref (recipientClass, preferenceId, enclosingUI);
      if (pref != null)
	     return pref;
      else if (subordinate != null)
         return subordinate. getPref (recipientClass, preferenceId, enclosingUI);
      else return null;
   }
   
   /** <p>Invokes {@link #getPref(Field, Pref, UI)} but the target receiver (field 
       or class), and the enclosing UI are obtained from the target receiver "ui".</p>
   */
   
   public Object getPref (UI ui, Pref preferenceId) {
      if (ui. recipientField != null) 
         return getPref (ui. recipientField, preferenceId, ui. enclosingUI);
      else return getPref (ui. recipientClass, preferenceId, ui. enclosingUI);
   }
   
   /** <p>Same as {@link #getPref(Field, Pref, UI)} except that field and class searches  
       are omitted, only default searches are made.  This factory is searched first, then its 
       subordinate if needed.</p>
   */
   
   public Object getPref (Pref preferenceId) {
      Object pref = defaultPrefs (). get (preferenceId);
      if (pref != null)
	     return pref;
      else if (subordinate != null)
         return subordinate. defaultPrefs (). get (preferenceId);
      else return null;
   }
   
   
   /** <p>Same as {@link #getPref(Field, Pref, UI)}, but the preference value is boolean.</p>
   */
   
   public boolean getBooleanPref (Field field, Pref preferenceId, UI enclosingUI) {
      Boolean b = (Boolean) getPref (field, preferenceId, enclosingUI);
      if (b == null)
         return false;
      else return b. booleanValue ();
   }
   
   /** <p>Same as {@link #getPref(UI, Pref)} but the preference value is boolean.</p> */
   
   public boolean getBooleanPref (UI ui, Pref preferenceId) {
      Boolean b = (Boolean) getPref (ui, preferenceId);
      if (b == null)
         return false;
      else return b. booleanValue ();
   }
   
   /** <p>Same as {@link #getPref(Field, Pref, UI)}, but the preference value is int.</p>
   */

   public int getIntPref (Field field, Pref preferenceId, UI enclosingUI) {
      Integer i = (Integer) getPref (field, preferenceId, enclosingUI);
      if (i == null)
         throw new Error ("No default available for preference " + preferenceId. getName () + ".");
      else return i. intValue ();
   }
   
   /** <p>Same as {@link #getPref(Class, Pref, UI)}, but the preference value is int.</p>
   */
   
   public int getIntPref (Class recipientClass, Pref preferenceId, UI enclosingUI) {
      Integer i = (Integer) getPref (recipientClass, preferenceId, enclosingUI);
      if (i == null)
         throw new Error ("No default available for preference " + preferenceId. getName () + ".");
      else return i. intValue ();
   }
   
   /** <p>Same as {@link #getPref(UI, Pref)} but the preference value is int.</p> */
   
   public int getIntPref (UI ui, Pref preferenceId) {
      Integer i = (Integer) getPref (ui, preferenceId);
      if (i == null)
         throw new Error ("No default available for preference " + preferenceId. getName () + ".");
      else return i. intValue ();
   }

   /** <p>This method adds the preferences from another factory to this factory, 
       without replacing any pre-existing preferences in this factory.</p>
   */
   
   public void addAllPrefsNoReplace (UIFactory anotherFactory) {
      if (! anotherFactory. fieldPrefs (). isEmpty ())
         fieldPrefs(). putAllNoReplace (anotherFactory. fieldPrefs());
      if (! anotherFactory. classPrefs (). isEmpty ())
         classPrefs(). putAllNoReplace (anotherFactory. classPrefs());
      if (! anotherFactory. defaultPrefs (). isEmpty ())
         defaultPrefs(). putAllNoReplace (anotherFactory. defaultPrefs());
   }
   
   /** <p>Same as {@link #getPref(Field, Pref, UI)} but it performs the search within the 
       generality levels of this factory only and does not go to this factory's subordinate 
       factory.</p>
   */
   
   protected Object getUnsubordinatedPref (Field field, Pref preferenceId, UI enclosingUI) {
      
      Hashtable prefs = (Hashtable) fieldPrefs (). get (field);
      if (prefs == null)
         return getUnsubordinatedPref (field. getType (), preferenceId, enclosingUI);
      else {
         Object pref = prefs. get (preferenceId);
         if (pref != null)
         	return pref;
         else return getUnsubordinatedPref (field. getType (), preferenceId, enclosingUI);
      }
   }
   
   /** <p>Same as {@link #getPref(Class, Pref, UI)} but it performs the search within the 
       generality levels of this factory only and does not go to this factory's subordinate 
       factory.  This is the "funnel" class search method used by others and where the super 
       class and contextual class searches are actually implemented.  The "parent" is 
       used to go up the object structure during contextual searches, and if it is null, then 
       the contextual search is not performed.</p>
   */

   protected Object getUnsubordinatedPref (Class recipientClass, Pref preferenceId, UI parent) {
      
      while (recipientClass != Object. class && recipientClass != null) {
         Hashtable prefs = (Hashtable) classPrefs (). get (recipientClass);
         if (prefs != null) {
            Object pref = prefs. get (preferenceId);
            if (pref != null)
               return pref;
         }
         if (cascadeToSuperClasses)
            recipientClass = recipientClass. getSuperclass ();
         
         if (cascadeToEnclosingObjects && preferenceId. isInheritable () && parent != null
         	 && (! cascadeToSuperClasses || recipientClass == Object. class || recipientClass == null)) 
         {
         	/* We are done going up the super class chain for this class, 
         	   so go up to the enclosing object's class so that we can traverse 
         	   all its superclasses as well.  This is a double loop, the inner 
         	   loop goes through the super class chain.  The outer loop goes 
         	   through the enclosing object structure chain.
         	*/
         	
         	recipientClass = parent. recipientClass;
            parent = parent. enclosingUI;
         }
         else return defaultPrefs (). get (preferenceId);
      }
      return defaultPrefs (). get (preferenceId);
   }
   
   /*---------------- End Cascading Preference Methods ----------------*/
   
   /*----------------- Begin UI Construction Methods ------------------*/
   
   /** <p>This convenience method returns true if there is a cascading preference  
       value for the preference Prefs. ELEMENTARY_UI_CLASS.</p>
   */
   
   public boolean hasElementaryUI (Class recipientClass) {
      Hashtable prefsTable; 
      if (subordinate == null) 
         prefsTable = (Hashtable) classPrefs (). get (recipientClass);
      else prefsTable = (Hashtable) subordinate. classPrefs (). get (recipientClass);
      return (prefsTable != null && prefsTable. get (Prefs. ELEMENTARY_UI_CLASS) != null);
   }
   
   /** <p>Convenience method to obtain the most preferred elementary UI for this field.  
       The preference Prefs.ELEMENTARY_UI_CLASS is retrieved, instantiated, initialized,  
       and the newly created object returned.</p>
   */
   
   public ElementaryUI createElementaryUI (Field field, CompositeUI parentUI) {
      Class euiClass = (Class) getPref (field, Prefs. ELEMENTARY_UI_CLASS, null);
      try {
         ElementaryUI eui = (ElementaryUI) euiClass. newInstance ();
         eui. init (field, parentUI);
         return eui;
      }
      catch (Exception e) { throw new Error (e. getMessage ()); }
   }
   
   /** <p>Equivalent to {@link #createElementaryUI(Field, CompositeUI)} except that the 
       recipient is not a field in the enclosing UI, and therefore the recipient object 
       is used instead.</p>
   */
   
   public ElementaryUI createElementaryUI (Object recipient, String simpleName, CompositeUI parentUI) {
      Class euiClass = (Class) getPref (recipient. getClass (), Prefs. ELEMENTARY_UI_CLASS, null);
      try {
         ElementaryUI eui = (ElementaryUI) euiClass. newInstance ();
         eui. init (recipient, simpleName, parentUI);
         return eui;
      }
      catch (Exception e) { throw new Error (e. getMessage ()); }
   }
   
   /** <p>Convenience method to create the most preferred composite UI for this field. The 
       preference Prefs.COMPOSITE_UI_CLASS is retrieved and used to create a composite UI 
       for this field.  The resulting composite UI is not initialized.  "field" may not be 
       null.</p>
   */

   public CompositeUI createCompositeUI (Field field) {
      Class cuiClass = (Class) getPref (field, Prefs. COMPOSITE_UI_CLASS, null);
      try {
         CompositeUI cui = (CompositeUI) cuiClass. newInstance ();
         cui. recipientClass = field. getType ();
         return cui;
      }
      catch (Exception e) { throw new Error (e. getMessage ()); }
   }
   
   /** <p>Convenience method to create the most preferred composite UI for this class. The 
       preference Prefs.COMPOSITE_UI_CLASS is retrieved and used to create a composite UI 
       for this class.  The resulting composite UI is not initialized.  "recipientClass" 
       may not be null.</p>
   */
	
   public CompositeUI createCompositeUI (Class recipientClass) {
      Class cuiClass;
   	  if (recipientClass == null)
   	     cuiClass = (Class) getPref (Prefs. COMPOSITE_UI_CLASS);
   	  else cuiClass = (Class) getPref (recipientClass, Prefs. COMPOSITE_UI_CLASS, null);
   	  
      try {
         CompositeUI cui = (CompositeUI) cuiClass. newInstance ();
         cui. recipientClass = recipientClass;
         return cui;
      }
      catch (Exception e) { throw new Error (e. getMessage ()); }
   }
   
   /*------------------ End UI Construction Methods -------------------*/

   /* End of UIFactory */
}