/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum.prefs;

import java.lang.reflect.Field;

import com.eknowlogie.aliorsum.UI;
import com.eknowlogie.aliorsum.UIFactory;

   /** <p>This is the Display enumeration preference and it is of the same enumeration type 
       as the {@link Access} preference.  It can only take on the values HIDDEN, READ_ONLY, 
       READ_AND_WRITE.  It is used to describe the maximum level of allowed visibility that  
       can be used when displaying the recipient according to its access privilege.  It is 
       used in conjunction with the Access preference.  While Access is used to specify the 
       privilege with which a user or situation is allowed to access a recipient object, 
       Display is used to denote the display or visibility level with which these access 
       privileges can be presented to the user. While conceptually both Access and Display 
       are the same, having two preferences instead of one allows us greater control in 
       customizing presentations.  For example, we can use the same page to allow the customer 
       to input a purchase order and to display it to shipping for fulfillment, by simply 
       changing the local preference default for Display from READ_AND_WRITE to READ_ONLY while 
       preserving access privileges for specific fields, such as hiding the order number from 
       the customer and hiding the credit card number from shipping.</p>
       
       <p>Contrary to <code>Access</code>, <code>Display</code> is an inheritable preference.</p>
    
       @author Romel Rivera, eKnowlogie. Inc.
   */

public class Display extends Access {
	
   /*-------------------------- Begin Fields --------------------------*/
	
   private final Access access;

   /*--------------------------- End Fields ---------------------------*/
  
   /*----------------------- Begin Constructors -----------------------*/
   
   /** <p>Parameterless constructor for this inheritable preference.</p> */
   
   public Display (Access access) { 
      super ("DISPLAY", true); 
      this. access = access;
   }
   
   /*------------------------ End Constructors ------------------------*/

   /*-------------------------- Begin Methods -------------------------*/
   
   /** <p>This method returns true if the value of this cascading preference for 
       preference recipient "field" is HIDDEN, false otherwise.</p>
       
       @see UIFactory#getPref(Field, Pref, UI)
   */
   
   public boolean isHidden (Field field, UI enclosingUI) { 
      int accessValue = enclosingUI. getFactory (). getIntPref (field, this, enclosingUI);
      return accessValue == HIDDEN. intValue ();
   }

   /** <p>This method returns true if the value of combined preference ACCESS  
       and DISPLAY for preference recipient at "ui" results in READ_ONLY, false 
       otherwise.</p>
   */
   
   public boolean isReadOnly (UI ui) {
      int accessValue = ui. getFactory (). getIntPref (ui, access);
      int displayValue = ui. getFactory (). getIntPref (ui, this);
      return isReadOnly (displayValue, accessValue);
   }
   
   /** <p>This method returns true if the combination of preferences display and 
       access result in a read only setting.</p>
   */
   
   private boolean isReadOnly (int displayValue, int accessValue) {
      return ((accessValue == READ_ONLY. intValue ()) 
	           && ((displayValue == READ_ONLY. intValue ())
	              || (displayValue == READ_AND_WRITE. intValue ())))
			 || ((accessValue == READ_AND_WRITE. intValue ()) 
			     && (displayValue == READ_ONLY. intValue ()));
   }
   
   /*--------------------------- End Methods --------------------------*/
   
   /* End of Display */
   
}
