/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum.prefs;

import java.lang.reflect.Field;

import com.eknowlogie.aliorsum.UI;
import com.eknowlogie.aliorsum.UIFactory;

/** <p>This class represents a Gaviidae preference.  An instance of this class serves as a 
    "preference name" or <b>preference identifier</b> for that preference and <b>does not</b> 
    hold preference values.  The alternative, naming preferences (properties) with anonymous 
    typeless textual strings, though a pervasive practice, circumvents Java encapsulation, 
    referencial and type-checking mechanisms already awarded by the language, and is critical  
    to the maintainability of source code. This class will evolve 1) to further characterize 
    the behavior of preferences, including support for both inherited and synthesized preference 
    propagation through recipient object structure trees, 2) to offer strong typing mechanisms 
    for corresponding preference values, and 3) to provide convenience methods to retrieve and 
    test its values, which are currently still being performed in class Prefs.</p>
    
    <p>Preference values are selected through a cascading preference selection mechanism 
    implemented by a factory (see {@link com.eknowlogie.aliorsum.UIFactory}).  A preference 
    value typically affects or is targeted for a specific recipient represented by a recipient 
    field, recipient class, or the recipient at a given UI.</p>
    
    @author Romel Rivera, eKnowlogie. Inc.
*/

public class Pref {
	
   /*-------------------------- Begin Fields --------------------------*/
	
   /** <p>If true, this flag indicates that a preference for composite recipient object, 
       can be propagated to its fields.  A read only display preference is inheritable, 
       if shipping address of type Address is read only, it follows that its zip code is 
       also read only, unless explicitly stated otherwise.  A title preference is not 
       inheritable.  A title "Shipping Address for All Purchases" for a shipping address 
       cannot be used to label its zip code.</p>
   */
	
   private boolean inheritable = false;
   
   /** <p>This is the readable name we assign to this preference id, to facilitate the 
       creation of readable reports of exception messages for the user.  It is not used 
       functionally in any way.</p>
   */
   
   private String name = null;
        
   /*--------------------------- End Fields ---------------------------*/
   
   /*----------------------- Begin Constructors -----------------------*/
   
   /** <p>Constructor for non-inheritable preferences.</p> 
   
       @see #Pref(String, boolean)
   */
   
   public Pref (String name) { this. name = name; }
   
   /** <p>Constructor to specify if this preference is inheritable or not.</p> 
    
       @param name <p>This is the readable name we assign to this preference id, to 
                   facilitate the creation of readable exception messages for the user.  
                   It is not used functionally in any way.</p>
                   
       @param inheritable See {@link #isInheritable()}
   */
   
   public Pref (String name, boolean inheritable) { 
      this (name);
      this. inheritable = inheritable; 
   }
   
   /*------------------------ End Constructors ------------------------*/

   /*-------------------------- Begin Methods -------------------------*/
   
   
   /** <p>If true, this flag indicates that a preference for a composite recipient object, 
       can be propagated to its fields.  A read only display preference is inheritable; 
       if shipping address of type Address is read only, it follows that its zip code is 
       also read only, unless explicitly stated otherwise.  A title preference is not 
       inheritable; a title "Shipping Address for All Purchases" for a shipping address 
       cannot be used to label its zip code.</p>
   */
   
   public boolean isInheritable () {
      return inheritable;
   }
   
   /** <p>This method returns the string name for this preference id.</p> */
   
   public String getName () {
      return name;
   }
   
   /** <p>This method uses cascade preference selection to find this preference value 
       for the given recipient at "ui".  Under Java 5.0 generics, the return type will be 
       a generic parameter so that it can be replaced during extensions of this class and 
       this method can be overriden without requiring type coercion on the return.  In 
       the meantime, instead of overriding it, we use different method names in the extensions 
       for this class.</p>
   */ 
   
   public Object get (UI ui) {
      return ui. getFactory (). getPref (ui, this);
   }
   
   /** <p>This method uses cascade preference selection to find this preference value 
       for the given field inside the enclosing UI.  Under Java 5.0 generics, the return 
       type will be a generic parameter so that it can be replaced during extensions of 
       this class and this method can be overriden without requiring type coercion on the 
       return.  In the meantime, instead of overriding it, we use different method names 
       in the extensions for this class.</p>
       
       @see UIFactory#getPref(Field, Pref, UI)
   */
   
   public Object get (Field field, UI enclosingUI) { 
      return enclosingUI. getFactory (). getPref (field, this, enclosingUI);
   }
   
   /*--------------------------- End Methods --------------------------*/
   
   /* End of Pref */
   
}
