/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum.prefs;

import java.util.Vector;

import com.eknowlogie.aliorsum.*;

/** <p>This class is the home for the Gaviidae <b>core cascading preferences</b>.  This class   
    provides a constant final id with which to refer or "name" each preference.  These preference 
    id's can only be initialized once and must remain constant since they are used as the keys 
    for all their preference values registered in factories throughout the application.  These 
    preference id's are not declared as final so that they can be extended. Gaviidae supports 
    the creation of arbitrary preferences, but unlike the pervasive use of anonymous property 
    string names, the Gaviidae preferences are typed (see {@link Pref}).  It is best to group 
    preferences and make them available to developers through theme preference classes like 
    this one (e.g. formatting or security preferences) or simply by extending this class. By 
    using theme preference classes, these preferences can be given a unique global Java constant 
    id which allows reference localization throughout the application, and the values assigned 
    can also be type-checked. Javadoc documentation can then be provided as the centralized 
    repository where the nature and purpose of these preferences is described. These classes 
    are also used to register the default values for these preferences.</p>
    
    <p><b>Multi-Valued Preference Flags</b>.  Under Java 5.0, all multi-valued preference 
    flags should be declared as enumeration types to further support strong typing through 
    convenience methods for their registration and access. Having all theme preferences 
    situated in their own class also makes it easy to later perform optimized registration 
    and searching.  A subordinate default is required for these preferences.</p>
    
    <p><b>Boolean Preferences</b>.  If a boolean preference is not found, then its 
    value is undefined, like any other preference, and it is not assumed to be 
    false, which allows for the cascading preference selection mechanism to continue.  
    A subordinate default is required for these preferences.</p>
    
    @author Romel Rivera, eKnowlogie. Inc.
*/

public class Prefs {

   /*-------------------------- Begin Fields --------------------------*/
	
   /** <p>This is the id of the Access preference.</p> */

   public static Access ACCESS;
   
   /** <p>This is the if of the Display inheritable enumeration preference and it is of the  
       same enumeration type as the ACCESS preference.</p>
   */

   public static Display DISPLAY;

   /** <p>This is the id of the WHEN_INVALID boolean preference.  It is applicable only 
       after a validation iteration has produced validation errors requiring the same form 
       to be redisplayed so show the validation messages.  If true, it means that the recipient 
       object should be displayed only if its previous user input was invalid.  If false, 
       then the recipient is redisplayed whether its value was valid or invalid.  This 
       preference is inheritable so it can be assigned to a composite object with many 
       elementary and composite subfields.  Typically, this preference will be registered 
       as a default activity preference so that it can be applied to an entire activity. 
       This preference is useful during the input of large forms in order to reduce clutter 
       and allow the user to easily locate invalid input.  Gaviidae provides a capability 
       in a composite UI to allow the user to toggle between full display and invalid-only 
       display of forms (recipients) during validation iterations.</p>
   */
   
   public static BooleanPref WHEN_INVALID;
   
   /** <p>This is the id of the IS_REQUIRED boolean preference. If true, the preference 
       recipient is a required object and must be input.  If false, it is optional.  As 
       with all boolean preferences, it must have a default otherwise its value would 
       be unspecified as it does not default to false.</p>
   */
   
   public static BooleanPref IS_REQUIRED;
   
   /** <p>This is the id of the TITLE String preference.</p> */
   
   public static Title TITLE;
   
   /** <p>This is the id of the POP_UP String preference. A value of this preference 
       refers to an Html page which is to be used as a pop-up window to provide explanations 
       about the recipient when the user clicks at the title of the recipient.  This preference 
       is not inheritable.  See "aliorsum/PopUp.html" for an example of a pop-up Html page.</p>
   */
   
   public static Pref POP_UP;
   
   /** <p>This is the id of the ToolTip String preference.</p> */
   
   public static ToolTip TOOLTIP;

   /** <p>This is the name of the COMPOSITE_UI_CLASS preference of type Class.  This
       preference must be the CompositeUI class or subclass, and it must have a 
       subordinate default.  An instance of this class is created by a UI factory, in 
       order to manage the presentation for the preference recipient.</p>
       
       <p>The relationship between UI classes, recipient classes and JSP renderers are 
       all many to many.  In fact, different UI classes and JSP renderers could be 
       selected dynamically for different instances of the same recipient class. When 
       appropiate, UI classes and JSP renderers can be bound to a specific recipient 
       class.  The matching between UI's, recipients and renderers can be done through 
       cascading preferences or dynamically through Java code in UI subclasses or 
       elsewhere.</p>
   */
   
   public static Pref COMPOSITE_UI_CLASS;
   
   /** <p>This is the name of the ELEMENTARY_UI_CLASS preference of type Class.  This
       preference must be an implementation of ElementaryUI or a subclass, and it must 
       have a subordinate default, although it is required that all primitive Java types 
       have an explicit ELEMENTARY_UI_CLASS subordinate default preference.  An instance 
       of this class is created by a UI factory, in order to manage the elementary 
       presentation for the preference recipient.  For details about the relationship 
       between UI classes, recipients and renderers, see {@link #COMPOSITE_UI_CLASS}.</p>
   */
   
   public static Pref ELEMENTARY_UI_CLASS;
   
   /** <p>This is the name of the RENDERER String preference.  A value for this preference 
       is the name of a JSP page or include file which is to be used as the renderer for 
       its composite recipient.  A subordinate default for this preference is required.  For 
       details about the relationship between UI classes, recipients and renderers, see 
       {@link #COMPOSITE_UI_CLASS}.</p>
   */
   
   public static Pref RENDERER;
   
   /** <p>This variable helps insure that these preferences are only initialized once.</p> */
   
   private static boolean initialized = false;
   
   /*--------------------------- End Fields ---------------------------*/

   /*---------------------- Begin Static Methods ----------------------*/

   /** <p>This method registers the subordinate defaults for the core cascading 
       preferences.  A default is a subordinate default if it is registered at 
       the lowest level or subordinate factory, which is typically the session 
       factory.</p>
   */
   
   protected void registerDefaults (UIFactory uif) {
   	
   	  uif. defaultPrefs (). put (ACCESS, Access. READ_AND_WRITE);
   	  uif. defaultPrefs (). put (DISPLAY, Display. READ_AND_WRITE);
   	  uif. defaultPrefs (). put (IS_REQUIRED, new Boolean (true));
   	  uif. defaultPrefs (). put (COMPOSITE_UI_CLASS, CompositeUI. class);
   	  uif. defaultPrefs (). put (ELEMENTARY_UI_CLASS, ElementaryUI. class);
   	  uif. defaultPrefs (). put (RENDERER, "UIRenderer.jsp");
   	  uif. classPrefs (). register (Vector. class, COMPOSITE_UI_CLASS, ListUI. class);
   	  uif. classPrefs (). register (Vector. class, RENDERER, "TableUIRendererForList.jsp");
   }
   
   /** <p>This method initializes all the preference id's.  Preference id's 
       can only be initialized once.  They are meant to be treated as constants 
       but not defined as final so that they can be extended.</p>
   */
   
   public void initializePrefs (UIFactory uif) {
   
      if (initialized) 
         throw new Error ("Gaviidae preferences can only be initialized once.");
      else initialized = true;
      
      ACCESS = new Access ("ACCESS");
      DISPLAY = new Display (ACCESS);
      WHEN_INVALID = new BooleanPref ("WHEN_INVALID", true);
      IS_REQUIRED = new BooleanPref ("IS_REQUIRED");
      TITLE = new Title ();
      POP_UP = new Pref ("POP_UP");
      TOOLTIP = new ToolTip ();
      COMPOSITE_UI_CLASS = new Pref ("COMPOSITE_UI_CLASS");
      ELEMENTARY_UI_CLASS = new Pref ("ELEMENTARY_UI_CLASS");
      RENDERER = new Pref ("RENDERER");
      
      registerDefaults (uif);
   }
   
   /*----------------------- End Static Methods -----------------------*/

   /* End of Prefs */
}