/* 
   (C) Copyright 2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.aliorsum.prefs;

import java.lang.reflect.Field;
import com.eknowlogie.util.Strings;
import com.eknowlogie.aliorsum.UI;

   /** <p>This is the ToolTip String preference. A value of this preference is the (Html) 
       text which is to be shown in a pop-up tooltip box to provide explanations about 
       the recipient.  This preference is not inheritable.  A tooltip text can embed the 
       tooltip text of the next superclass with a tooltip preference, by embedding the  
       string tag <code>{&#64;inherit}</code> in its text.  If there is no superclass with  
       a tooltip, then the embedded tag is simply ignored and removed from the text.  When 
       rendering the tooltip, a UI will typically add Html formatting tags as well as 
       additional messages, e.g. to explain if this recipient is required.</p>
    
       @author Romel Rivera, eKnowlogie. Inc.
   */

public class ToolTip extends Pref {
	
   /*-------------------------- Begin Fields --------------------------*/
	      
   /** <p>This is the string used in embedded tooltip messages to inherit the tooltip 
       message from the recipient's super class.</p>
   */
   
   private static String inheritToolTipCmd = "{@inherit}";
   
   /*--------------------------- End Fields ---------------------------*/
   
   /*----------------------- Begin Constructors -----------------------*/
   
   
   /** <p>This is the only parameterless constructor needed.</p> */
   
   public ToolTip () { super ("ToolTip", false); }
   
   /*------------------------ End Constructors ------------------------*/

   /*-------------------------- Begin Methods -------------------------*/
   
   /** <p>This method uses cascade preference selection to find the tooltip string  
       for this UI.  The returned preference value is the (html) text to be shown in 
       the tooltip pop-up box when the user moves the mouse over the title of this 
       recipient.  This method supports embedded inheritance of tool tips for the super 
       classes of recipient.</p>
       
       @see Pref#get(UI)
   */
   
   public String getToolTip (UI ui) {
      return getToolTip ((String) super. get (ui), ui. recipientClass, ui);
   }
   
   /** <p>This method returns a tooltip preference value as described in {@link #getToolTip(UI)}, 
       except that the tooltip is for the specified orphan field, that is, a field that is not 
       necessarily directly enclosed in this UI's recipient.  An enclosing UI is not needed in 
       this situation since none of the preferences needed to render a title are inheritable.</p>
   */
   
   public String getToolTip (Field field, UI unrelatedUI) {
      return getToolTip ((String) super. get (field, unrelatedUI), field. getType (), unrelatedUI);
   }
   
   /** <p>This method implements {@link #getToolTip(UI)}.</p> */
   
   protected String getToolTip (String toolTipText, Class superClass, UI unrelatedUI) {

      if (toolTipText == null)
         return null;
      
      String previousTipText = toolTipText;
      boolean inheritSuperTip = toolTipText. indexOf (inheritToolTipCmd) != -1;
      while (inheritSuperTip && superClass != null && superClass != Object. class) {
         String superTipText = (String) unrelatedUI. getFactory (). getPref (superClass, this, unrelatedUI);
         if (superTipText == previousTipText) 
            superClass = superClass. getSuperclass ();
         else {
            previousTipText = superTipText;
            if (superTipText == null)
            	superTipText = "";
            toolTipText= Strings. findAndReplace (toolTipText, inheritToolTipCmd, superTipText);
            superClass = superClass. getSuperclass ();
            inheritSuperTip = toolTipText. indexOf (inheritToolTipCmd) != -1;
         }
      }
      return toolTipText;
   }
   
   
   /*--------------------------- End Methods --------------------------*/
   
   /* End of ToolTip */
   
}
