/* 
   (C) Copyright 2002-2004 eKnowlogie Inc., All rights reserved.
   eKnowlogie Proprietary / Confidential.  Use is subject to license terms.
*/

package com.eknowlogie.util;

import java.util.*;

/** <p>This is hash table but its creation is deferred until it is really needed,
    in order to save space.  This class is convenient for pervasive use, for example,
    to represent node attributes in every single node in a tree.</p>
    
    @author Romel Rivera, eKnowlogie. Inc.
*/

public class DeferredHashTable implements Cloneable {

   /*-------------------------- Begin Fields --------------------------*/

   /** <p>It is intialized to null to avoid taking memory space when not
       in use.  Attributes are similar to properties, except their keys
       are always Strings, case-sensitive, and their values can
       be anything.  Attributes are not persistent.  For standardization, modularization
       and safety, the names of these attributes should always be supplied in the
       form of string constants by the action class which implements these attributes.
       So for example, formatting attribute names should be supplied by a tree formatting
       class.  File include attributes should be supplied by the include file super
       class, etc. </p>
   */

   public Hashtable table = null;

   /*--------------------------- End Fields ---------------------------*/

   /*----------------------- Begin Constructors -----------------------*/

   public DeferredHashTable () {
   }

   /*------------------------ End Constructors ------------------------*/

   /*---------------------- Begin Static Methods ----------------------*/

   public static Object clone (Object attributesObj) {
      try {
         if (attributesObj == null)
            return null;
         else return ((DeferredHashTable) attributesObj). clone ();
      }
      catch (CloneNotSupportedException e) { throw new Error (e. getMessage()); }
   }

   /*---------------------- End Static Methods ------------------------*/

   /*------------------------- Begin Methods --------------------------*/
   
   public boolean isEmpty () {
     return table == null || table. isEmpty ();
   }
   
   public Set entrySet () {
      if (table == null)
         return new HashSet ();
      else return table. entrySet ();
   }

   public Object get (Object key) {
      if (table == null)
         return null;
      return table. get (key);
   }
   
   public boolean getBoolean (Object key) {
      Boolean b = (Boolean) get (key);
      if (b == null)
         return false;
      else return b. booleanValue ();
   }
   
   public int getInt (Object key) {
      Integer i = (Integer) get (key);
      if (i == null)
         return -1;
      else return i. intValue ();
   }

   public boolean containsKey (Object key) {
      if (table == null)
         return false;
      return table. containsKey (key);
   }

   public Object put (Object key, Object value) {
      if (table == null)
         table = new Hashtable ();
      return table. put (key, value);
   }

   public void putAll (Object otherTable) {
      if (otherTable == null)
         return;
      else if (table == null)
         table = new Hashtable ();
      table. putAll (((DeferredHashTable) otherTable). table);
   }
   
   public void putAllNoReplace (DeferredHashTable otherTable) {
      if (otherTable. isEmpty ())
         return;
      else {
         Iterator it = otherTable. entrySet (). iterator ();
         while (it. hasNext ()) {
            Map. Entry entry = (Map. Entry) it. next ();
            if (! containsKey (entry. getKey ()))
               put (entry. getKey (), entry. getValue ());
         }
      }
   }

   public Object remove (Object key) {
      if (table == null)
         return null;
      return table. remove (key);
   }

   public Object clone () throws CloneNotSupportedException {
      DeferredHashTable replica = (DeferredHashTable) super. clone ();
      if (table != null)
         replica. table = (Hashtable) table. clone ();
      return replica;
   }

   /*-------------------------- End Methods ---------------------------*/

   /* End of DeferredHashTable */
}
